@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 ##
-# Copyright 2013-2017 Ghent University
+# Copyright 2013-2019 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -9,7 +9,7 @@
 # Flemish Research Foundation (FWO) (http://www.fwo.be/en)
 # and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
 #
-# http://github.com/hpcugent/easybuild
+# https://github.com/easybuilders/easybuild
 #
 # EasyBuild is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -49,11 +49,12 @@ import site
 import sys
 import tempfile
 import traceback
+import urllib2
 from distutils.version import LooseVersion
 from hashlib import md5
 
 
-EB_BOOTSTRAP_VERSION = '20161109.01'
+EB_BOOTSTRAP_VERSION = '20190112.01'
 
 # argparse preferrred, optparse deprecated >=2.7
 HAVE_ARGPARSE = False
@@ -69,6 +70,8 @@ VSC_BASE = 'vsc-base'
 VSC_INSTALL = 'vsc-install'
 EASYBUILD_PACKAGES = [VSC_INSTALL, VSC_BASE, 'easybuild-framework', 'easybuild-easyblocks', 'easybuild-easyconfigs']
 
+STAGE1_SUBDIR = 'eb_stage1'
+
 # set print_debug to True for detailed progress info
 print_debug = os.environ.pop('EASYBUILD_BOOTSTRAP_DEBUG', False)
 
@@ -84,6 +87,7 @@ os.environ['PYTHONPATH'] = ''
 
 EASYBUILD_BOOTSTRAP_SOURCEPATH = os.environ.pop('EASYBUILD_BOOTSTRAP_SOURCEPATH', None)
 EASYBUILD_BOOTSTRAP_SKIP_STAGE0 = os.environ.pop('EASYBUILD_BOOTSTRAP_SKIP_STAGE0', False)
+EASYBUILD_BOOTSTRAP_FORCE_VERSION = os.environ.pop('EASYBUILD_BOOTSTRAP_FORCE_VERSION', None)
 
 # keep track of original environment (after clearing PYTHONPATH)
 orig_os_environ = copy.deepcopy(os.environ)
@@ -92,6 +96,10 @@ orig_os_environ = copy.deepcopy(os.environ)
 easybuild_modules_tool = os.environ.get('EASYBUILD_MODULES_TOOL', None)
 easybuild_module_syntax = os.environ.get('EASYBUILD_MODULE_SYNTAX', None)
 
+# If modules subdir specifications are defined, use them
+easybuild_installpath_modules = os.environ.get('EASYBUILD_INSTALLPATH_MODULES', None)
+easybuild_subdir_modules = os.environ.get('EASYBUILD_SUBDIR_MODULES', 'modules')
+easybuild_suffix_modules_path = os.environ.get('EASYBUILD_SUFFIX_MODULES_PATH', 'all')
 
 #
 # Utility functions
@@ -152,6 +160,16 @@ def det_lib_path(libdir):
     return os.path.join(libdir, 'python%s' % pyver, 'site-packages')
 
 
+def det_modules_path(install_path):
+    """Determine modules path."""
+    if easybuild_installpath_modules is not None:
+        modules_path = os.path.join(easybuild_installpath_modules, easybuild_suffix_modules_path)
+    else:
+        modules_path = os.path.join(install_path, easybuild_subdir_modules, easybuild_suffix_modules_path)
+
+    return modules_path
+
+
 def find_egg_dir_for(path, pkg):
     """Find full path of egg dir for given package."""
 
@@ -224,23 +242,35 @@ def check_module_command(tmpdir):
 
     def check_cmd_help(modcmd):
         """Check 'help' output for specified command."""
-        modcmd_re = re.compile(r'module\s.*command\s')
+        modcmd_re = re.compile(r'module\s.*command')
         cmd = "%s python help" % modcmd
         os.system("%s > %s 2>&1" % (cmd, out))
         txt = open(out, 'r').read()
         debug("Output from %s: %s" % (cmd, txt))
         return modcmd_re.search(txt)
 
+    def is_modulecmd_tcl_modulestcl():
+        """Determine if modulecmd.tcl is EnvironmentModulesTcl."""
+        modcmd_re = re.compile('Modules Release Tcl')
+        cmd = "modulecmd.tcl python --version"
+        os.system("%s > %s 2>&1" % (cmd, out))
+        txt = open(out, 'r').read()
+        debug("Output from %s: %s" % (cmd, txt))
+        return modcmd_re.search(txt)
+
     # order matters, which is why we don't use a dict
     known_module_commands = [
-        ('modulecmd', 'EnvironmentModulesC'),
         ('lmod', 'Lmod'),
-        ('modulecmd.tcl', 'EnvironmentModulesTcl'),
+        ('modulecmd.tcl', 'EnvironmentModules'),
+        ('modulecmd', 'EnvironmentModulesC'),
     ]
     out = os.path.join(tmpdir, 'module_command.out')
     modtool = None
     for modcmd, modtool in known_module_commands:
         if check_cmd_help(modcmd):
+            # distinguish between EnvironmentModulesTcl and EnvironmentModules
+            if modcmd == 'modulecmd.tcl' and is_modulecmd_tcl_modulestcl():
+                modtool = 'EnvironmentModulesTcl'
             easybuild_modules_tool = modtool
             info("Found module command '%s' (%s), so using it." % (modcmd, modtool))
             break
@@ -251,6 +281,14 @@ def check_module_command(tmpdir):
                 easybuild_modules_tool = modtool
                 info("Found module command '%s' via $LMOD_CMD (%s), so using it." % (modcmd, modtool))
                 break
+        elif modtool == 'EnvironmentModules':
+            # check value of $MODULESHOME as fallback
+            moduleshome = os.environ.get('MODULESHOME', 'MODULESHOME_NOT_DEFINED')
+            modcmd = os.path.join(moduleshome, 'libexec', 'modulecmd.tcl')
+            if os.path.exists(modcmd) and check_cmd_help(modcmd):
+                easybuild_modules_tool = modtool
+                info("Found module command '%s' via $MODULESHOME (%s), so using it." % (modcmd, modtool))
+                break
 
     if easybuild_modules_tool is None:
         mod_cmds = [m for (m, _) in known_module_commands]
@@ -322,7 +360,7 @@ def run_easy_install(args):
     try:
         easy_install.main(args)
         easy_install_stdout, easy_install_stderr = restore_stdout_stderr(orig_stdout, orig_stderr)
-    except Exception as err:
+    except (Exception, SystemExit) as err:
         easy_install_stdout, easy_install_stderr = restore_stdout_stderr(orig_stdout, orig_stderr)
         error("Running 'easy_install %s' failed: %s\n%s" % (' '.join(args), err, traceback.format_exc()))
 
@@ -343,26 +381,16 @@ def check_easy_install_cmd():
     easy_install_regex = re.compile('^(setuptools|distribute) %s' % setuptools.__version__)
     debug("Pattern for 'easy_install --version': %s" % easy_install_regex.pattern)
 
-    for path in os.getenv('PATH', '').split(os.pathsep):
-        easy_install = os.path.join(path, 'easy_install')
-        debug("Checking %s..." % easy_install)
-        res = False
-        if os.path.exists(easy_install):
-            cmd = "PYTHONPATH='%s' %s %s --version" % (os.getenv('PYTHONPATH', ''), sys.executable, easy_install)
-            os.system("%s > %s 2>&1" % (cmd, outfile))
-            outtxt = open(outfile).read().strip()
-            debug("Output of '%s':\n%s" % (cmd, outtxt))
-            res = bool(easy_install_regex.match(outtxt))
-            debug("Result for %s: %s" % (easy_install, res))
-        else:
-            debug("%s does not exist" % easy_install)
-
-        if res:
-            debug("Found right 'easy_install' command in %s" % path)
-            curr_path = os.environ.get('PATH', '').split(os.pathsep)
-            os.environ['PATH'] = os.pathsep.join([path] + curr_path)
-            debug("$PATH: %s" % os.environ['PATH'])
-            return
+    pythonpath = os.getenv('PYTHONPATH', '')
+    cmd = "PYTHONPATH='%s' %s -m easy_install --version" % (pythonpath, sys.executable)
+    os.system("%s > %s 2>&1" % (cmd, outfile))
+    outtxt = open(outfile).read().strip()
+    debug("Output of '%s':\n%s" % (cmd, outtxt))
+    res = bool(easy_install_regex.match(outtxt))
+    debug("Result: %s" % res)
+    if res:
+        debug("Found right 'easy_install' command")
+        return
 
     error("Failed to find right 'easy_install' command!")
 
@@ -405,7 +433,7 @@ def stage0(tmpdir):
     # We download a custom version of distribute: it uses a newer version of markerlib to avoid a bug (#1099)
     # It's is the source of distribute 0.6.49 with the file _markerlib/markers.py replaced by the 0.6 version of
     # markerlib which can be found at https://pypi.python.org/pypi/markerlib/0.6.0
-    sys.argv.append('--download-base=http://hpcugent.github.io/easybuild/files/')
+    sys.argv.append('--download-base=https://easybuilders.github.io/easybuild/files/')
     distribute_setup_main(version="0.6.49-patched1")
     sys.argv = orig_sys_argv
 
@@ -437,7 +465,7 @@ def stage0(tmpdir):
     return distribute_egg_dir
 
 
-def stage1(tmpdir, sourcepath, distribute_egg_dir):
+def stage1(tmpdir, sourcepath, distribute_egg_dir, forcedversion):
     """STAGE 1: temporary install EasyBuild using distribute's easy_install."""
 
     print('\n')
@@ -468,7 +496,7 @@ def stage1(tmpdir, sourcepath, distribute_egg_dir):
         run_easy_install(['--help'])
 
     # prepare install dir
-    targetdir_stage1 = os.path.join(tmpdir, 'eb_stage1')
+    targetdir_stage1 = os.path.join(tmpdir, STAGE1_SUBDIR)
     prep(targetdir_stage1)  # set PATH, Python search path
 
     # install latest EasyBuild with easy_install from PyPi
@@ -486,7 +514,10 @@ def stage1(tmpdir, sourcepath, distribute_egg_dir):
             cmd.append(source_tarballs[VSC_BASE])
     else:
         # install meta-package easybuild from PyPI
-        cmd.append('easybuild')
+        if forcedversion:
+            cmd.append('easybuild==%s' % forcedversion)
+        else:
+            cmd.append('easybuild')
 
         # install vsc-base again at the end, to avoid that the one available on the system is used instead
         post_vsc_base = cmd[:]
@@ -495,6 +526,10 @@ def stage1(tmpdir, sourcepath, distribute_egg_dir):
     if not print_debug:
         cmd.insert(0, '--quiet')
 
+    # install vsc-install version prior to 0.11.4, where mock was introduced as a dependency
+    # workaround for problem reported in https://github.com/easybuilders/easybuild-framework/issues/2712
+    run_easy_install(cmd[:-1] + [VSC_INSTALL + '<0.11.4'])
+
     info("installing EasyBuild with 'easy_install %s'" % (' '.join(cmd)))
     run_easy_install(cmd)
 
@@ -600,39 +635,73 @@ def stage2(tmpdir, templates, install_path, distribute_egg_dir, sourcepath):
     preinstallopts = ''
 
     if distribute_egg_dir is not None:
-        # inject path to distribute installed in stage 1 into $PYTHONPATH via preinstallopts
+        # inject path to distribute installed in stage 0 into $PYTHONPATH via preinstallopts
         # other approaches are not reliable, since EasyBuildMeta easyblock unsets $PYTHONPATH;
-        # this is required for the easy_install from stage 1 to work
+        # this is required for the easy_install from stage 0 to work
         preinstallopts += "export PYTHONPATH=%s:$PYTHONPATH && " % distribute_egg_dir
 
         # ensure that (latest) setuptools is installed as well alongside EasyBuild,
         # since it is a required runtime dependency for recent vsc-base and EasyBuild versions
         # this is necessary since we provide our own distribute installation during the bootstrap (cfr. stage0)
-        preinstallopts += "%s $(which easy_install) -U --prefix %%(installdir)s setuptools && " % sys.executable
+        preinstallopts += "%s -m easy_install -U --prefix %%(installdir)s setuptools && " % sys.executable
 
     # vsc-install is a runtime dependency for the EasyBuild unit test suite,
     # and is easily picked up from stage1 rather than being actually installed, so force it
-    vsc_install = 'vsc-install'
+    vsc_install = "'%s<0.11.4'" % VSC_INSTALL
     if sourcepath:
         vsc_install_tarball_paths = glob.glob(os.path.join(sourcepath, 'vsc-install*.tar.gz'))
         if len(vsc_install_tarball_paths) == 1:
             vsc_install = vsc_install_tarball_paths[0]
-    preinstallopts += "%s $(which easy_install) -U --prefix %%(installdir)s %s && " % (sys.executable, vsc_install)
+    preinstallopts += "%s -m easy_install -U --prefix %%(installdir)s %s && " % (sys.executable, vsc_install)
 
     templates.update({
         'preinstallopts': preinstallopts,
     })
 
-    # create easyconfig file
-    ebfile = os.path.join(tmpdir, 'EasyBuild-%s.eb' % templates['version'])
-    handle = open(ebfile, 'w')
+    # determine PyPI URLs for individual packages
+    pkg_urls = []
+    for pkg in EASYBUILD_PACKAGES:
+        # format of pkg entries in templates: "'<pkg_filename>',"
+        pkg_filename = templates[pkg][1:-2]
+
+        # the lines below implement a simplified version of the 'pypi_source_urls' and 'derive_alt_pypi_url' functions,
+        # which we can't leverage here, partially because of transitional changes in PyPI (#md5= -> #sha256=)
+
+        # determine download URL via PyPI's 'simple' API
+        pkg_simple = None
+        try:
+            pkg_simple = urllib2.urlopen('https://pypi.python.org/simple/%s' % pkg, timeout=10).read()
+        except (urllib2.URLError, urllib2.HTTPError) as err:
+            # failing to figure out the package download URl may be OK when source tarballs are provided
+            if sourcepath:
+                info("Ignoring failed attempt to determine '%s' download URL since source tarballs are provided" % pkg)
+            else:
+                raise err
+
+        if pkg_simple:
+            pkg_url_part_regex = re.compile('/(packages/[^#]+)/%s#' % pkg_filename)
+            res = pkg_url_part_regex.search(pkg_simple)
+            if res:
+                pkg_url_part = res.group(1)
+            else:
+                error("Failed to determine PyPI package URL for %s: %s\n" % (pkg, pkg_simple))
+
+            pkg_url = 'https://pypi.python.org/' + pkg_url_part
+            pkg_urls.append(pkg_url)
+
     templates.update({
-        'source_urls': '\n'.join(["'%s/%s/%s'," % (PYPI_SOURCE_URL, pkg[0], pkg) for pkg in EASYBUILD_PACKAGES]),
+        'source_urls': '\n'.join(["'%s'," % pkg_url for pkg_url in pkg_urls]),
         'sources': "%(vsc-install)s%(vsc-base)s%(easybuild-framework)s%(easybuild-easyblocks)s%(easybuild-easyconfigs)s" % templates,
         'pythonpath': distribute_egg_dir,
     })
-    handle.write(EASYBUILD_EASYCONFIG_TEMPLATE % templates)
+
+    # create easyconfig file
+    ebfile = os.path.join(tmpdir, 'EasyBuild-%s.eb' % templates['version'])
+    handle = open(ebfile, 'w')
+    ebfile_txt = EASYBUILD_EASYCONFIG_TEMPLATE % templates
+    handle.write(ebfile_txt)
     handle.close()
+    debug("Contents of generated easyconfig file:\n%s" % ebfile_txt)
 
     # set command line arguments for eb
     eb_args = ['eb', ebfile, '--allow-modules-tool-mismatch']
@@ -660,7 +729,7 @@ def stage2(tmpdir, templates, install_path, distribute_egg_dir, sourcepath):
 
     # make sure parent modules path already exists (Lmod trips over a non-existing entry in $MODULEPATH)
     if install_path is not None:
-        modules_path = os.path.join(install_path, 'modules', 'all')
+        modules_path = det_modules_path(install_path)
         if not os.path.exists(modules_path):
             os.makedirs(modules_path)
         debug("Created path %s" % modules_path)
@@ -668,10 +737,51 @@ def stage2(tmpdir, templates, install_path, distribute_egg_dir, sourcepath):
     debug("Running EasyBuild with arguments '%s'" % ' '.join(eb_args))
     sys.argv = eb_args
 
+    # location to 'eb' command (from stage 1) may be expected to be included in $PATH
+    # it usually is there after stage1, unless 'prep' is called again with another location
+    # (only when stage 0 is not skipped)
+    # cfr. https://github.com/easybuilders/easybuild-framework/issues/2279
+    curr_path = [x for x in os.environ.get('PATH', '').split(os.pathsep) if len(x) > 0]
+    os.environ['PATH'] = os.pathsep.join([os.path.join(tmpdir, STAGE1_SUBDIR, 'bin')] + curr_path)
+    debug("$PATH: %s" % os.environ['PATH'])
+
     # install EasyBuild with EasyBuild
     from easybuild.main import main as easybuild_main
     easybuild_main()
 
+    if print_debug:
+        os.environ['EASYBUILD_DEBUG'] = '1'
+
+    # make sure the EasyBuild module was actually installed
+    # EasyBuild configuration options that are picked up from configuration files/environment may break the bootstrap,
+    # for example by having $EASYBUILD_VERSION defined or via a configuration file specifies a value for 'stop'...
+    from easybuild.tools.config import build_option, install_path, get_module_syntax
+    from easybuild.framework.easyconfig.easyconfig import ActiveMNS
+    eb_spec = {
+        'name': 'EasyBuild',
+        'hidden': False,
+        'toolchain': {'name': 'dummy', 'version': 'dummy'},
+        'version': templates['version'],
+        'versionprefix': '',
+        'versionsuffix': '',
+        'moduleclass': 'tools',
+    }
+
+    mod_path = os.path.join(install_path('mod'), build_option('suffix_modules_path'))
+    debug("EasyBuild module should have been installed to %s" % mod_path)
+
+    eb_mod_name = ActiveMNS().det_full_module_name(eb_spec)
+    debug("EasyBuild module name: %s" % eb_mod_name)
+
+    eb_mod_path = os.path.join(mod_path, eb_mod_name)
+    if get_module_syntax() == 'Lua':
+        eb_mod_path += '.lua'
+
+    if os.path.exists(eb_mod_path):
+        info("EasyBuild module installed: %s" % eb_mod_path)
+    else:
+        error("EasyBuild module not found at %s, define $EASYBUILD_BOOTSTRAP_DEBUG to debug" % eb_mod_path)
+
 
 def main():
     """Main script: bootstrap EasyBuild in stages."""
@@ -713,6 +823,10 @@ def main():
     if sourcepath is not None:
         info("Fetching sources from %s..." % sourcepath)
 
+    forcedversion = EASYBUILD_BOOTSTRAP_FORCE_VERSION
+    if forcedversion:
+        info("Forcing specified version %s..." % forcedversion)
+
     # create temporary dir for temporary installations
     tmpdir = tempfile.mkdtemp()
     debug("Going to use %s as temporary directory" % tmpdir)
@@ -759,7 +873,7 @@ def main():
             distribute_egg_dir = stage0(tmpdir)
 
     # STAGE 1: install EasyBuild using easy_install to tmp dir
-    templates = stage1(tmpdir, sourcepath, distribute_egg_dir)
+    templates = stage1(tmpdir, sourcepath, distribute_egg_dir, forcedversion)
 
     # add location to easy_install provided through stage0 to $PATH
     # this must be done *after* stage1, since $PATH is reset during stage1
@@ -781,7 +895,7 @@ def main():
 
     if install_path is not None:
         info('EasyBuild v%s was installed to %s, so make sure your $MODULEPATH includes %s' %
-             (templates['version'], install_path, os.path.join(install_path, 'modules', 'all')))
+             (templates['version'], install_path, det_modules_path(install_path)))
     else:
         info('EasyBuild v%s was installed to configured install path, make sure your $MODULEPATH is set correctly.' %
              templates['version'])
@@ -802,7 +916,7 @@ easyblock = 'EB_EasyBuildMeta'
 name = 'EasyBuild'
 version = '%(version)s'
 
-homepage = 'http://hpcugent.github.com/easybuild/'
+homepage = 'http://easybuilders.github.com/easybuild/'
 description = \"\"\"EasyBuild is a software build and installation framework
 written in Python that allows you to install software in a structured,
 repeatable and robust way.\"\"\"
@@ -816,7 +930,7 @@ sources = [%(sources)s]
 # usually, we want to use the system Python, so no actual Python dependency is listed
 allow_system_deps = [('Python', SYS_PYTHON_VERSION)]
 
-preinstallopts = '%(preinstallopts)s'
+preinstallopts = "%(preinstallopts)s"
 
 pyshortver = '.'.join(SYS_PYTHON_VERSION.split('.')[:2])
 sanity_check_paths = {
