@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 ##
-# Copyright 2009-2017 Ghent University
+# Copyright 2009-2019 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -9,7 +9,7 @@
 # Flemish Research Foundation (FWO) (http://www.fwo.be/en)
 # and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
 #
-# http://github.com/hpcugent/easybuild
+# https://github.com/easybuilders/easybuild
 #
 # EasyBuild is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -56,7 +56,7 @@ if not len(args) == 1:
     sys.exit(1)
 
 name = args[0]
-print "Template easyblock for %s requested..." % name
+print("Template easyblock for %s requested..." % name)
 
 # check whether easyblock repository path is found
 easyblocks_repo_path = os.path.join(options.path, "easybuild", "easyblocks")
@@ -98,7 +98,7 @@ tmpl = """##
 # Flemish Research Foundation (FWO) (http://www.fwo.be/en)
 # and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
 #
-# http://github.com/hpcugent/easybuild
+# https://github.com/easybuilders/easybuild
 #
 # EasyBuild is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -221,7 +221,7 @@ txt = tmpl % {
               'parent': options.parent,
              }
 
-print "Writing template easyblock for %s to %s ..." % (name, easyblock_path)
+print("Writing template easyblock for %s to %s ..." % (name, easyblock_path))
 try:
     dirpath = os.path.dirname(easyblock_path)
     if not os.path.exists(dirpath):
@@ -229,6 +229,6 @@ try:
     f = open(easyblock_path, "w")
     f.write(txt)
     f.close()
-except (IOError, OSError), err:
+except (IOError, OSError) as err:
     sys.stderr.write("ERROR! Writing template easyblock for %s to %s failed: %s" % (name, easyblock_path, err))
     sys.exit(1)
