@@ -1,5 +1,5 @@
 ##
-# Copyright 2012-2017 Ghent University
+# Copyright 2012-2019 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -8,7 +8,7 @@
 # Flemish Research Foundation (FWO) (http://www.fwo.be/en)
 # and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
 #
-# http://github.com/hpcugent/easybuild
+# https://github.com/easybuilders/easybuild
 #
 # EasyBuild is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@ EasyBuild support for gompi compiler toolchain (includes GCC and OpenMPI).
 
 :author: Kenneth Hoste (Ghent University)
 """
+from distutils.version import LooseVersion
 
 from easybuild.toolchains.gcc import GccToolchain
 from easybuild.toolchains.mpi.openmpi import OpenMPI
@@ -36,3 +37,13 @@ class Gompi(GccToolchain, OpenMPI):
     """Compiler toolchain with GCC and OpenMPI."""
     NAME = 'gompi'
     SUBTOOLCHAIN = GccToolchain.NAME
+
+    def is_deprecated(self):
+        """Return whether or not this toolchain is deprecated."""
+        # deprecate oldest gompi toolchains (versions 1.x)
+        if LooseVersion(self.version) < LooseVersion('2000'):
+            deprecated = True
+        else:
+            deprecated = False
+
+        return deprecated
