@@ -1,5 +1,5 @@
 ##
-# Copyright 2012-2017 Ghent University
+# Copyright 2012-2019 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -8,7 +8,7 @@
 # Flemish Research Foundation (FWO) (http://www.fwo.be/en)
 # and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
 #
-# http://github.com/hpcugent/easybuild
+# https://github.com/easybuilders/easybuild
 #
 # EasyBuild is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@ EasyBuild support for intel compiler toolchain (includes Intel compilers (icc, i
 :author: Stijn De Weirdt (Ghent University)
 :author: Kenneth Hoste (Ghent University)
 """
+from distutils.version import LooseVersion
 
 from easybuild.toolchains.iccifort import IccIfort
 from easybuild.toolchains.mpi.intelmpi import IntelMPI
@@ -39,3 +40,17 @@ class Iimpi(IccIfort, IntelMPI):
     """
     NAME = 'iimpi'
     SUBTOOLCHAIN = IccIfort.NAME
+
+    def is_deprecated(self):
+        """Return whether or not this toolchain is deprecated."""
+        # iimpi toolchains older than iimpi/2016.01 are deprecated
+        # iimpi 8.1.5 is an exception, since it used in intel/2016a (which is not deprecated yet)
+        iimpi_ver = LooseVersion(self.version)
+        if iimpi_ver < LooseVersion('8.0'):
+            deprecated = True
+        elif iimpi_ver > LooseVersion('2000') and iimpi_ver < LooseVersion('2016.01'):
+            deprecated = True
+        else:
+            deprecated = False
+
+        return deprecated
