@@ -1,5 +1,5 @@
 ##
-# Copyright 2012-2017 Ghent University
+# Copyright 2012-2019 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -8,7 +8,7 @@
 # Flemish Research Foundation (FWO) (http://www.fwo.be/en)
 # and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
 #
-# http://github.com/hpcugent/easybuild
+# https://github.com/easybuilders/easybuild
 #
 # EasyBuild is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,8 +28,13 @@ Support for OpenMPI as toolchain MPI library.
 :author: Stijn De Weirdt (Ghent University)
 :author: Kenneth Hoste (Ghent University)
 """
+import os
+import shutil
+import tempfile
 from distutils.version import LooseVersion
 
+import easybuild.tools.environment as env
+from easybuild.tools.build_log import print_warning
 from easybuild.tools.toolchain.constants import COMPILER_VARIABLES, MPI_COMPILER_VARIABLES
 from easybuild.tools.toolchain.mpi import Mpi
 from easybuild.tools.toolchain.variables import CommandFlagList
@@ -57,6 +62,30 @@ class OpenMPI(Mpi):
 
     MPI_LINK_INFO_OPTION = '-showme:link'
 
+    def __init__(self, *args, **kwargs):
+        """Toolchain constructor"""
+        super(OpenMPI, self).__init__(*args, **kwargs)
+
+        self.orig_tmpdir = None
+
+    def prepare(self, *args, **kwargs):
+        """
+        Prepare for using OpenMPI library in toolchain environment
+        """
+        super(OpenMPI, self).prepare(*args, **kwargs)
+
+        # OpenMPI 2.x trips if path specified in $TMPDIR is too long
+        # see https://www.open-mpi.org/faq/?category=osx#startup-errors-with-open-mpi-2.0.x
+        self.orig_tmpdir = os.environ.get('TMPDIR')
+        ompi_ver = self.get_software_version(self.MPI_MODULE_NAME)[0]
+        if LooseVersion(ompi_ver) >= LooseVersion('2.0') and LooseVersion(ompi_ver) < LooseVersion('3.0'):
+            if len(self.orig_tmpdir) > 40:
+                tmpdir = tempfile.mkdtemp(prefix='/tmp/')
+                env.setvar('TMPDIR', tmpdir)
+                warn_msg = "Long $TMPDIR path may cause problems with OpenMPI 2.x, using shorter path: %s" % tmpdir
+                self.log.warning(warn_msg)
+                print_warning(warn_msg)
+
     def _set_mpi_compiler_variables(self):
         """Define MPI wrapper commands (depends on OpenMPI version) and add OMPI_* variables to set."""
         ompi_ver = self.get_software_version(self.MPI_MODULE_NAME)[0]
@@ -75,3 +104,16 @@ class OpenMPI(Mpi):
             self.variables.nappend('OMPI_%s' % var, str(self.variables[var].get_first()), var_class=CommandFlagList)
 
         super(OpenMPI, self)._set_mpi_compiler_variables()
+
+    def cleanup(self, *args, **kwargs):
+        """Clean up after using OpenMPI in toolchain."""
+        super(OpenMPI, self).cleanup(*args, **kwargs)
+
+        tmpdir = os.environ.get('TMPDIR')
+        if tmpdir != self.orig_tmpdir:
+            try:
+                shutil.rmtree(tmpdir)
+            except OSError as err:
+                print_warning("Failed to clean up temporary directory %s: %s", tmpdir, err)
+            env.setvar('TMPDIR', self.orig_tmpdir)
+            self.log.info("$TMPDIR restored to %s", self.orig_tmpdir)
