@@ -1,5 +1,5 @@
 # #
-# Copyright 2009-2017 Ghent University
+# Copyright 2009-2019 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -8,7 +8,7 @@
 # Flemish Research Foundation (FWO) (http://www.fwo.be/en)
 # and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
 #
-# http://github.com/hpcugent/easybuild
+# https://github.com/easybuilders/easybuild
 #
 # EasyBuild is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -37,9 +37,7 @@ Documentation-related functionality
 import copy
 import inspect
 import os
-import re
 import string
-import sys
 from distutils.version import LooseVersion
 from vsc.utils import fancylogger
 from vsc.utils.docs import mk_rst_table
@@ -48,13 +46,14 @@ from vsc.utils.missing import nub
 from easybuild.framework.easyconfig.default import DEFAULT_CONFIG, HIDDEN, sorted_categories
 from easybuild.framework.easyblock import EasyBlock
 from easybuild.framework.easyconfig.constants import EASYCONFIG_CONSTANTS
-from easybuild.framework.easyconfig.easyconfig import EasyConfig, get_easyblock_class, process_easyconfig
+from easybuild.framework.easyconfig.easyconfig import get_easyblock_class, process_easyconfig
 from easybuild.framework.easyconfig.licenses import EASYCONFIG_LICENSES_DICT
 from easybuild.framework.easyconfig.parser import EasyConfigParser
 from easybuild.framework.easyconfig.templates import TEMPLATE_NAMES_CONFIG, TEMPLATE_NAMES_EASYCONFIG
 from easybuild.framework.easyconfig.templates import TEMPLATE_NAMES_LOWER, TEMPLATE_NAMES_LOWER_TEMPLATE
 from easybuild.framework.easyconfig.templates import TEMPLATE_NAMES_EASYBLOCK_RUN_STEP, TEMPLATE_CONSTANTS
 from easybuild.framework.easyconfig.templates import TEMPLATE_SOFTWARE_VERSIONS, template_constant_dict
+from easybuild.framework.easyconfig.tools import avail_easyblocks
 from easybuild.framework.easyconfig.tweak import find_matching_easyconfigs
 from easybuild.framework.extension import Extension
 from easybuild.tools.build_log import EasyBuildError, print_msg
@@ -123,12 +122,12 @@ def avail_cfgfile_constants_txt(go_cfg_constants):
 
 def avail_cfgfile_constants_rst(go_cfg_constants):
     title = "Constants available (only) in configuration files"
-    doc =[title, '-' * len(title), '']
+    doc = [title, '-' * len(title), '']
 
     for section in go_cfg_constants:
         doc.append('')
         if section != go_cfg_constants['DEFAULT']:
-            section_title = "only in '%s' section:" %section
+            section_title = "only in '%s' section:" % section
             doc.extend([section_title, '-' * len(section_title), ''])
         table_titles = ["Constant name", "Constant help", "Constant value"]
         table_values = [
@@ -183,7 +182,7 @@ def avail_easyconfig_licenses(output_format=FORMAT_TXT):
 def avail_easyconfig_licenses_txt():
     """Generate easyconfig license documentation in txt format"""
     doc = ["License constants that can be used in easyconfigs"]
-    for lic_name, lic in EASYCONFIG_LICENSES_DICT.items():
+    for lic_name, lic in sorted(EASYCONFIG_LICENSES_DICT.items()):
         lic_inst = lic()
         strver = ''
         if lic_inst.version:
@@ -203,10 +202,11 @@ def avail_easyconfig_licenses_rst():
         "Version",
     ]
 
+    lics = sorted(EASYCONFIG_LICENSES_DICT.items())
     table_values = [
-        ["``%s``" % name for name in EASYCONFIG_LICENSES_DICT.keys()],
-        ["%s" % lic().description for lic in EASYCONFIG_LICENSES_DICT.values()],
-        ["``%s``" % lic().version for lic in EASYCONFIG_LICENSES_DICT.values()],
+        ["``%s``" % lic().name for _, lic in lics],
+        ["%s" % lic().description for _, lic in lics],
+        ["``%s``" % lic().version for _, lic in lics],
     ]
 
     doc = rst_title_and_table(title, table_titles, table_values)
@@ -274,7 +274,7 @@ def avail_easyconfig_params(easyblock, output_format=FORMAT_TXT):
 
     # include list of extra parameters (if any)
     extra_params = {}
-    app = get_easyblock_class(easyblock, default_fallback=False)
+    app = get_easyblock_class(easyblock, error_on_missing_easyblock=False)
     if app is not None:
         extra_params = app.extra_options()
     params.update(extra_params)
@@ -418,9 +418,9 @@ def avail_classes_tree(classes, class_names, locations, detailed, format_strings
             mod = class_info['module']
             loc = ''
             if mod in locations:
-                loc = '@ %s' % locations[mod]
+                loc = '@ %s' % locations[mod]['loc']
             txt.append(format_strings['zero_indent'] + format_strings['indent'] * depth +
-                        format_strings['sep'] + "%s (%s %s)" % (class_name, mod, loc))
+                       format_strings['sep'] + "%s (%s %s)" % (class_name, mod, loc))
         else:
             txt.append(format_strings['zero_indent'] + format_strings['indent'] * depth +
                        format_strings['sep'] + class_name)
@@ -428,7 +428,8 @@ def avail_classes_tree(classes, class_names, locations, detailed, format_strings
             if len(class_info['children']) > 0:
                 if format_strings.get('newline') is not None:
                     txt.append(format_strings['newline'])
-                txt.extend(avail_classes_tree(classes, class_info['children'], locations, detailed, format_strings, depth + 1))
+                txt.extend(avail_classes_tree(classes, class_info['children'], locations, detailed,
+                                              format_strings, depth + 1))
                 if format_strings.get('newline') is not None:
                     txt.append(format_strings['newline'])
     return txt
@@ -436,14 +437,14 @@ def avail_classes_tree(classes, class_names, locations, detailed, format_strings
 
 def list_easyblocks(list_easyblocks=SIMPLE, output_format=FORMAT_TXT):
     format_strings = {
-        FORMAT_TXT : {
+        FORMAT_TXT: {
             'det_root_templ': "%s (%s%s)",
             'root_templ': "%s",
             'zero_indent': '',
             'indent': "|   ",
             'sep': "|-- ",
         },
-        FORMAT_RST : {
+        FORMAT_RST: {
             'det_root_templ': "* **%s** (%s%s)",
             'root_templ': "* **%s**",
             'zero_indent': INDENT_2SPACES,
@@ -458,38 +459,15 @@ def list_easyblocks(list_easyblocks=SIMPLE, output_format=FORMAT_TXT):
 def gen_list_easyblocks(list_easyblocks, format_strings):
     """Get a class tree for easyblocks."""
     detailed = list_easyblocks == DETAILED
-    module_regexp = re.compile(r"^([^_].*)\.py$")
 
-    # finish initialisation of the toolchain module (ie set the TC_CONSTANT constants)
-    search_toolchain('')
-
-    locations = {}
-    for package in ["easybuild.easyblocks", "easybuild.easyblocks.generic"]:
-        __import__(package)
-
-        # determine paths for this package
-        paths = sys.modules[package].__path__
-
-        # import all modules in these paths
-        for path in paths:
-            if os.path.exists(path):
-                for f in os.listdir(path):
-                    res = module_regexp.match(f)
-                    if res:
-                        easyblock = '%s.%s' % (package, res.group(1))
-                        if easyblock not in locations:
-                            __import__(easyblock)
-                            locations.update({easyblock: os.path.join(path, f)})
-                        else:
-                            _log.debug("%s already imported from %s, ignoring %s",
-                                               easyblock, locations[easyblock], path)
+    locations = avail_easyblocks()
 
     def add_class(classes, cls):
         """Add a new class, and all of its subclasses."""
         children = cls.__subclasses__()
         classes.update({cls.__name__: {
             'module': cls.__module__,
-            'children': [x.__name__ for x in children]
+            'children': sorted([c.__name__ for c in children], key=lambda x: x.lower())
         }})
         for child in children:
             add_class(classes, child)
@@ -509,7 +487,7 @@ def gen_list_easyblocks(list_easyblocks, format_strings):
             mod = classes[root]['module']
             loc = ''
             if mod in locations:
-                loc = ' @ %s' % locations[mod]
+                loc = ' @ %s' % locations[mod]['loc']
             txt.append(format_strings['det_root_templ'] % (root, mod, loc))
         else:
             txt.append(format_strings['root_templ'] % root)
@@ -558,20 +536,23 @@ def list_software(output_format=FORMAT_TXT, detailed=False, only_installed=False
         else:
             toolchain = '%s/%s' % (ec['toolchain']['name'], ec['toolchain']['version'])
 
-        versionsuffix = ec.get('versionsuffix', '')
+        keys = ['description', 'homepage', 'version', 'versionsuffix']
 
-        # make sure versionsuffix gets properly templated
-        if versionsuffix and isinstance(ec, dict):
-            template_values = template_constant_dict(ec)
-            versionsuffix = versionsuffix % template_values
+        info = {'toolchain': toolchain}
+        for key in keys:
+            info[key] = ec.get(key, '')
 
-        software[ec['name']].append({
-            'description': ec['description'],
-            'homepage': ec['homepage'],
-            'toolchain': toolchain,
-            'version': ec['version'],
-            'versionsuffix': versionsuffix,
-        })
+        # make sure values like homepage & versionsuffix get properly templated
+        if isinstance(ec, dict):
+            template_values = template_constant_dict(ec, skip_lower=False)
+            for key in keys:
+                if '%(' in info[key]:
+                    try:
+                        info[key] = info[key] % template_values
+                    except (KeyError, TypeError, ValueError) as err:
+                        _log.debug("Ignoring failure to resolve templates: %s", err)
+
+        software[ec['name']].append(info)
 
         if only_installed:
             software[ec['name']][-1].update({'mod_name': ec.full_mod_name})
@@ -667,7 +648,10 @@ def list_software_rst(software, detailed=False):
             for ver, vsuff in sorted((LooseVersion(v), vs) for (v, vs) in pairs):
                 table_values[0].append('``%s``' % ver)
                 if with_vsuff:
-                    table_values[1].append('``%s``' % vsuff)
+                    if vsuff:
+                        table_values[1].append('``%s``' % vsuff)
+                    else:
+                        table_values[1].append('')
                 tcs = [x['toolchain'] for x in software[key] if x['version'] == ver and x['versionsuffix'] == vsuff]
                 table_values[-1].append(', '.join('``%s``' % tc for tc in sorted(nub(tcs))))
 
@@ -781,7 +765,7 @@ def avail_toolchain_opts(name, output_format=FORMAT_TXT):
     tc_class, _ = search_toolchain(name)
     if not tc_class:
         raise EasyBuildError("Couldn't find toolchain: '%s'. To see available toolchains, use --list-toolchains" % name)
-    tc = tc_class(version='1.0') # version doesn't matter here, but needs to be defined
+    tc = tc_class(version='1.0')  # version doesn't matter here, but needs to be defined
 
     tc_dict = {}
     for cst in ['COMPILER_SHARED_OPTS', 'COMPILER_UNIQUE_OPTS', 'MPI_SHARED_OPTS', 'MPI_UNIQUE_OPTS']:
@@ -830,7 +814,7 @@ def gen_easyblocks_overview_rst(package_name, path_to_examples, common_params={}
 
     # get all blocks
     for mod in modules:
-        for name,obj in inspect.getmembers(mod, inspect.isclass):
+        for name, obj in inspect.getmembers(mod, inspect.isclass):
             eb_class = getattr(mod, name)
             # skip imported classes that are not easyblocks
             if eb_class.__module__.startswith(package_name) and eb_class not in all_blocks:
@@ -874,7 +858,7 @@ def gen_easyblock_doc_section_rst(eb_class, path_to_examples, common_params, doc
 
     bases = []
     for b in eb_class.__bases__:
-        base = ':ref:`' + b.__name__ +'`' if b in all_blocks else b.__name__
+        base = ':ref:`' + b.__name__ + '`' if b in all_blocks else b.__name__
         bases.append(base)
 
     derived = '(derives from ' + ', '.join(bases) + ')'
@@ -887,12 +871,14 @@ def gen_easyblock_doc_section_rst(eb_class, path_to_examples, common_params, doc
     if eb_class.extra_options():
         title = 'Extra easyconfig parameters specific to ``%s`` easyblock' % classname
         ex_opt = eb_class.extra_options()
+        keys = sorted(ex_opt.keys())
+        values = [ex_opt[k] for k in keys]
 
         table_titles = ['easyconfig parameter', 'description', 'default value']
         table_values = [
-            ['``' + key + '``' for key in ex_opt],  # parameter name
-            [val[1] for val in ex_opt.values()],  # description
-            ['``' + str(quote_str(val[0])) + '``' for val in ex_opt.values()]  # default value
+            ['``' + key + '``' for key in keys],  # parameter name
+            [val[1] for val in values],  # description
+            ['``' + str(quote_str(val[0])) + '``' for val in values]  # default value
         ]
 
         doc.extend(rst_title_and_table(title, table_titles, table_values))
