@@ -1,5 +1,5 @@
 ##
-# Copyright 2015-2017 Ghent University
+# Copyright 2015-2019 Ghent University
 # Copyright 2015 S3IT, University of Zurich
 #
 # This file is part of EasyBuild,
@@ -9,7 +9,7 @@
 # Flemish Research Foundation (FWO) (http://www.fwo.be/en)
 # and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
 #
-# http://github.com/hpcugent/easybuild
+# https://github.com/easybuilders/easybuild
 #
 # EasyBuild is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,13 +31,13 @@ Interface for submitting jobs via GC3Pie.
 """
 from distutils.version import LooseVersion
 from time import gmtime, strftime
-import re
 import time
 
+from pkg_resources import get_distribution, DistributionNotFound
 from vsc.utils import fancylogger
 
 from easybuild.tools.build_log import EasyBuildError, print_msg
-from easybuild.tools.config import build_option
+from easybuild.tools.config import JOB_DEPS_TYPE_ABORT_ON_ERROR, JOB_DEPS_TYPE_ALWAYS_RUN, build_option
 from easybuild.tools.job.backend import JobBackend
 from easybuild.tools.utilities import only_if_module_is_available
 
@@ -49,9 +49,8 @@ try:
     import gc3libs
     import gc3libs.exceptions
     from gc3libs import Application, Run, create_engine
-    from gc3libs.core import Engine
     from gc3libs.quantity import hours as hr
-    from gc3libs.workflow import DependentTaskCollection
+    from gc3libs.workflow import AbortOnError, DependentTaskCollection
 
     # inject EasyBuild logger into GC3Pie
     gc3libs.log = fancylogger.getLogger('gc3pie', fname=False)
@@ -61,6 +60,16 @@ try:
     # instruct GC3Pie to not ignore errors, but raise exceptions instead
     gc3libs.UNIGNORE_ALL_ERRORS = True
 
+    # note: order of class inheritance is important!
+    class AbortingDependentTaskCollection(AbortOnError, DependentTaskCollection):
+        """
+        A `DependentTaskCollection`:class: that aborts execution upon error.
+
+        This is used to stop the build process in case some dependency
+        fails.  See also `<https://github.com/easybuilders/easybuild-framework/issues/1441>`_
+        """
+        pass
+
 except ImportError as err:
     _log.debug("Failed to import gc3libs from GC3Pie."
                " Silently ignoring, this is a real issue only when GC3Pie is used as backend for --job")
@@ -78,8 +87,7 @@ class GC3Pie(JobBackend):
     terminated.
     """
 
-    REQ_VERSION = '2.4.0'
-    VERSION_REGEX = re.compile(r'^(?P<version>\S*) version')
+    REQ_VERSION = '2.5.0'
 
     @only_if_module_is_available('gc3libs', pkgname='gc3pie')
     def __init__(self, *args, **kwargs):
@@ -90,24 +98,15 @@ class GC3Pie(JobBackend):
     @only_if_module_is_available('gc3libs', pkgname='gc3pie')
     def _check_version(self):
         """Check whether GC3Pie version complies with required version."""
-        # location of __version__ to use may change, depending on the minimal required SVN revision for development versions
-        version_str = gc3libs.core.__version__
+        try:
+            pkg = get_distribution('gc3pie')
+        except DistributionNotFound as err:
+            raise EasyBuildError(
+                "Cannot load GC3Pie package: %s" % err)
 
-        match = self.VERSION_REGEX.search(version_str)
-        if match:
-            version = match.group('version')
-            self.log.debug("Parsed GC3Pie version info: '%s'", version)
-
-            if version == 'development':
-                # presume it's OK -- there's no way to check since GC3Pie switched to git
-                return True
-
-            if LooseVersion(version) < LooseVersion(self.REQ_VERSION):
-                raise EasyBuildError("Found GC3Pie version %s, but version %s or more recent is required",
-                                     version, self.REQ_VERSION)
-        else:
-            raise EasyBuildError("Failed to parse GC3Pie version string '%s' using pattern %s",
-                                 version_str, self.VERSION_REGEX.pattern)
+        if LooseVersion(pkg.version) < LooseVersion(self.REQ_VERSION):
+            raise EasyBuildError("Found GC3Pie version %s, but version %s or more recent is required",
+                                 pkg.version, self.REQ_VERSION)
 
     def init(self):
         """
@@ -124,9 +123,22 @@ class GC3Pie(JobBackend):
             self.config_files.append(cfgfile)
 
         self.output_dir = build_option('job_output_dir')
-        self.jobs = DependentTaskCollection(output_dir=self.output_dir)
         self.job_cnt = 0
 
+        job_deps_type = build_option('job_deps_type')
+        if job_deps_type is None:
+            job_deps_type = JOB_DEPS_TYPE_ABORT_ON_ERROR
+            self.log.info("Using default job dependency type: %s", job_deps_type)
+        else:
+            self.log.info("Using specified job dependency type: %s", job_deps_type)
+
+        if job_deps_type == JOB_DEPS_TYPE_ALWAYS_RUN:
+            self.jobs = DependentTaskCollection(output_dir=self.output_dir)
+        elif job_deps_type == JOB_DEPS_TYPE_ABORT_ON_ERROR:
+            self.jobs = AbortingDependentTaskCollection(output_dir=self.output_dir)
+        else:
+            raise EasyBuildError("Unknown job dependency type specified: %s", job_deps_type)
+
         # after polling for job status, sleep for this time duration
         # before polling again (in seconds)
         self.poll_interval = build_option('job_polling_interval')
@@ -135,27 +147,24 @@ class GC3Pie(JobBackend):
         """
         Create and return a job object with the given parameters.
 
-        First argument `server` is an instance of the corresponding
-        `JobBackend` class, i.e., a `GC3Pie`:class: instance in this case.
-
-        Second argument `script` is the content of the job script
+        Argument *script* is the content of the job script
         itself, i.e., the sequence of shell commands that will be
         executed.
 
-        Third argument `name` sets the job human-readable name.
+        Argument *name* sets the job's human-readable name.
 
-        Fourth (optional) argument `env_vars` is a dictionary with
+        Optional argument *env_vars* is a dictionary with
         key-value pairs of environment variables that should be passed
         on to the job.
 
-        Fifth and sixth (optional) arguments `hours` and `cores` should be
+        Optional arguments *hours* and *cores* should be
         integer values:
-        * hours must be in the range 1 .. MAX_WALLTIME;
-        * cores depends on which cluster the job is being run.
+        - *hours* must be in the range 1 .. ``MAX_WALLTIME``;
+        - *cores* depends on which cluster the job is being run.
         """
         named_args = {
-            'jobname': name, # job name in GC3Pie
-            'name':    name, # job name in EasyBuild
+            'jobname': name,  # job name in GC3Pie
+            'name':    name,  # job name in EasyBuild
         }
 
         # environment
@@ -219,12 +228,15 @@ class GC3Pie(JobBackend):
         # see https://gc3pie.readthedocs.org/en/latest/programmers/api/gc3libs/core.html#gc3libs.core.Engine
         self._engine.retrieve_overwrites = True
 
+        # some sites may not be happy with flooding the cluster with build jobs...
+        self._engine.max_in_flight = build_option('job_max_jobs')
+
         # Add your application to the engine. This will NOT submit
         # your application yet, but will make the engine *aware* of
         # the application.
         self._engine.add(self.jobs)
 
-        # in case you want to select a specific resource, call
+        # select a specific execution resource?
         target_resource = build_option('job_target_resource')
         if target_resource:
             res = self._engine.select_resource(target_resource)
@@ -253,10 +265,10 @@ class GC3Pie(JobBackend):
         Print a job status report to STDOUT and the log file.
 
         The number of jobs in each state is reported; the
-        figures are extracted from the `stats()` method of the
+        figures are extracted from the `counts()` method of the
         currently-running GC3Pie engine.
         """
-        stats = self._engine.stats(only=Application)
+        stats = self._engine.counts(only=Application)
         states = ', '.join(["%d %s" % (stats[s], s.lower()) for s in stats if s != 'total' and stats[s]])
         print_msg("GC3Pie job overview: %s (total: %s)" % (states, self.job_cnt),
                   log=self.log, silent=build_option('silent'))
