@@ -1,5 +1,5 @@
 # #
-# Copyright 2009-2017 Ghent University
+# Copyright 2009-2019 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -8,7 +8,7 @@
 # Flemish Research Foundation (FWO) (http://www.fwo.be/en)
 # and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
 #
-# http://github.com/hpcugent/easybuild
+# https://github.com/easybuilders/easybuild
 #
 # EasyBuild is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -48,8 +48,10 @@ from easybuild.tools.filetools import det_common_path_prefix, search_file
 from easybuild.tools.module_naming_scheme.easybuild_mns import EasyBuildMNS
 from easybuild.tools.module_naming_scheme.utilities import det_full_ec_version
 
+
 _log = fancylogger.getLogger('tools.robot', fname=False)
 
+
 def det_robot_path(robot_paths_option, tweaked_ecs_paths, pr_path, auto_robot=False):
     """Determine robot path."""
     robot_path = robot_paths_option[:]
@@ -91,14 +93,35 @@ def check_conflicts(easyconfigs, modtool, check_inter_ec_conflicts=True):
 
         return (spec['name'], det_full_ec_version(spec))
 
+    # determine whether any 'wrappers' are involved
+    wrapper_deps = {}
+    for ec in ordered_ecs:
+        # easyconfigs using ModuleRC install a 'wrapper' for their dependency
+        # these need to be filtered out to avoid reporting false conflicts...
+        if ec['ec']['easyblock'] == 'ModuleRC':
+            wrapper_deps[mk_key(ec)] = mk_key(ec['ec']['dependencies'][0])
+
+    def mk_dep_keys(deps):
+        """Create keys for given list of dependencies."""
+        res = []
+        for dep in deps:
+            # filter out dependencies marked as external module
+            if not dep.get('external_module', False):
+                key = mk_key(dep)
+                # replace 'wrapper' dependencies with the dependency they're wrapping
+                if key in wrapper_deps:
+                    key = wrapper_deps[key]
+                res.append(key)
+        return res
+
     # construct a dictionary: (name, installver) tuple to (build) dependencies
     deps_for, dep_of = {}, {}
     for node in ordered_ecs:
         node_key = mk_key(node)
 
         # exclude external modules, since we can't check conflicts on them (we don't even know the software name)
-        build_deps = [mk_key(d) for d in node['builddependencies'] if not d.get('external_module', False)]
-        deps = [mk_key(d) for d in node['ec'].all_dependencies if not d.get('external_module', False)]
+        build_deps = mk_dep_keys(node['builddependencies'])
+        deps = mk_dep_keys(node['ec'].all_dependencies)
 
         # separate runtime deps from build deps
         runtime_deps = [d for d in deps if d not in build_deps]
@@ -111,8 +134,10 @@ def check_conflicts(easyconfigs, modtool, check_inter_ec_conflicts=True):
 
     if check_inter_ec_conflicts:
         # add ghost entry that depends on each of the specified easyconfigs,
-        # since we want to check for conflicts between specified easyconfigs too
-        deps_for[(None, None)] = ([], [mk_key(e) for e in easyconfigs])
+        # since we want to check for conflicts between specified easyconfigs too;
+        # 'wrapper' easyconfigs are not included to avoid false conflicts being reported
+        ec_keys = [k for k in [mk_key(e) for e in easyconfigs] if k not in wrapper_deps]
+        deps_for[(None, None)] = ([], ec_keys)
 
     # iteratively expand list of dependencies
     last_deps_for = None
@@ -192,7 +217,7 @@ def dry_run(easyconfigs, modtool, short=False):
         all_specs = easyconfigs
     else:
         lines.append("Dry run: printing build status of easyconfigs and dependencies")
-        all_specs = resolve_dependencies(easyconfigs, modtool, retain_all_deps=True)
+        all_specs = resolve_dependencies(easyconfigs, modtool, retain_all_deps=True, raise_error_missing_ecs=False)
 
     unbuilt_specs = skip_available(all_specs, modtool)
     dry_run_fmt = " * [%1s] %s (module: %s)"  # markdown compatible (list of items with checkboxes in front)
@@ -200,7 +225,7 @@ def dry_run(easyconfigs, modtool, short=False):
     listed_ec_paths = [spec['spec'] for spec in easyconfigs]
 
     var_name = 'CFGS'
-    common_prefix = det_common_path_prefix([spec['spec'] for spec in all_specs])
+    common_prefix = det_common_path_prefix([spec['spec'] for spec in all_specs if spec['spec'] is not None])
     # only allow short if common prefix is long enough
     short = short and common_prefix is not None and len(common_prefix) > len(var_name) * 2
     for spec in all_specs:
@@ -213,15 +238,18 @@ def dry_run(easyconfigs, modtool, short=False):
         else:
             ans = 'x'
 
-        if spec['ec'].short_mod_name != spec['ec'].full_mod_name:
+        if spec['ec'] is not None and spec['ec'].short_mod_name != spec['ec'].full_mod_name:
             mod = "%s | %s" % (spec['ec'].mod_subdir, spec['ec'].short_mod_name)
         else:
-            mod = spec['ec'].full_mod_name
+            mod = spec['full_mod_name']
 
-        if short:
+        if spec['spec'] is None:
+            item = "(no easyconfig file found)"
+        elif short:
             item = os.path.join('$%s' % var_name, spec['spec'][len(common_prefix) + 1:])
         else:
             item = spec['spec']
+
         lines.append(dry_run_fmt % (ans, item, mod))
 
     if short:
@@ -230,13 +258,30 @@ def dry_run(easyconfigs, modtool, short=False):
     return '\n'.join(lines)
 
 
-def resolve_dependencies(easyconfigs, modtool, retain_all_deps=False):
+def raise_error_missing_deps(missing_deps, extra_msg=None):
+    """Raise error to report missing dependencies."""
+
+    _log.warning("Missing dependencies (details): %s", missing_deps)
+
+    mod_names_eb = ', '.join(EasyBuildMNS().det_full_module_name(dep) for dep in missing_deps)
+    _log.warning("Missing dependencies (EasyBuild module names): %s", mod_names_eb)
+
+    mod_names = ', '.join(ActiveMNS().det_full_module_name(dep) for dep in missing_deps)
+
+    error_msg = "Missing dependencies: %s" % mod_names
+    if extra_msg:
+        error_msg += ' (%s)' % extra_msg
+    raise EasyBuildError(error_msg)
+
+
+def resolve_dependencies(easyconfigs, modtool, retain_all_deps=False, raise_error_missing_ecs=True):
     """
     Work through the list of easyconfigs to determine an optimal order
     :param easyconfigs: list of easyconfigs
     :param modtool: ModulesTool instance to use
     :param retain_all_deps: boolean indicating whether all dependencies must be retained, regardless of availability;
                             retain all deps when True, check matching build option when False
+    :param raise_error_missing_ecs: raise an error when one or more easyconfig files could not be found
     """
     robot = build_option('robot_path')
     # retain all dependencies if specified by either the resp. build option or the dedicated named argument
@@ -254,20 +299,21 @@ def resolve_dependencies(easyconfigs, modtool, retain_all_deps=False):
     ordered_ecs = []
     # all available modules can be used for resolving dependencies except those that will be installed
     being_installed = [p['full_mod_name'] for p in easyconfigs]
-    avail_modules = [m for m in avail_modules if not m in being_installed]
+    avail_modules = [m for m in avail_modules if m not in being_installed]
 
-    _log.debug('easyconfigs before resolving deps: %s' % easyconfigs)
+    _log.debug('easyconfigs before resolving deps: %s', easyconfigs)
+
+    totally_missing, missing_easyconfigs = [], []
 
     # resolve all dependencies, put a safeguard in place to avoid an infinite loop (shouldn't occur though)
-    irresolvable = []
     loopcnt = 0
     maxloopcnt = 10000
     while easyconfigs:
         # make sure this stops, we really don't want to get stuck in an infinite loop
         loopcnt += 1
         if loopcnt > maxloopcnt:
-            raise EasyBuildError("Maximum loop cnt %s reached, so quitting (easyconfigs: %s, irresolvable: %s)",
-                                 maxloopcnt, easyconfigs, irresolvable)
+            raise EasyBuildError("Maximum loop cnt %s reached, so quitting (easyconfigs: %s, missing_easyconfigs: %s)",
+                                 maxloopcnt, easyconfigs, missing_easyconfigs)
 
         # first try resolving dependencies without using external dependencies
         last_processed_count = -1
@@ -285,8 +331,8 @@ def resolve_dependencies(easyconfigs, modtool, retain_all_deps=False):
         missing_external_modules = [d['full_mod_name'] for ec in easyconfigs for d in ec['dependencies']
                                     if d.get('external_module', False)]
         if missing_external_modules:
-            raise EasyBuildError("Missing modules for one or more dependencies marked as external modules: %s",
-                                 missing_external_modules)
+            raise EasyBuildError("Missing modules for dependencies marked as external modules: %s",
+                                 ', '.join(missing_external_modules))
 
         # robot: look for existing dependencies, add them
         if robot and easyconfigs:
@@ -310,12 +356,29 @@ def resolve_dependencies(easyconfigs, modtool, retain_all_deps=False):
                     path = robot_find_easyconfig(cand_dep['name'], det_full_ec_version(cand_dep))
 
                     if path is None:
-                        # no easyconfig found for dependency, add to list of irresolvable dependencies
-                        if cand_dep not in irresolvable:
-                            _log.debug("Irresolvable dependency found: %s" % cand_dep)
-                            irresolvable.append(cand_dep)
+                        full_mod_name = ActiveMNS().det_full_module_name(cand_dep)
+
+                        # no easyconfig found + no module available => missing dependency
+                        if not modtool.exist([full_mod_name])[0]:
+                            if cand_dep not in totally_missing:
+                                totally_missing.append(cand_dep)
+
+                        # no easyconfig found for dependency, but module is available
+                        # => add to list of missing easyconfigs
+                        elif cand_dep not in missing_easyconfigs:
+                            _log.debug("Irresolvable dependency found (no easyconfig file): %s", cand_dep)
+                            missing_easyconfigs.append(cand_dep)
+
                         # remove irresolvable dependency from list of dependencies so we can continue
                         entry['dependencies'].remove(cand_dep)
+
+                        # add dummy entry for this dependency, so --dry-run for example can still report the dep
+                        additional.append({
+                            'dependencies': [],
+                            'ec': None,
+                            'full_mod_name': full_mod_name,
+                            'spec': None,
+                        })
                     else:
                         _log.info("Robot: resolving dependency %s with %s" % (cand_dep, path))
                         # build specs should not be passed down to resolved dependencies,
@@ -327,7 +390,7 @@ def resolve_dependencies(easyconfigs, modtool, retain_all_deps=False):
                         verify_easyconfig_filename(path, cand_dep, parsed_ec=processed_ecs)
 
                         for ec in processed_ecs:
-                            if not ec in easyconfigs + additional:
+                            if ec not in easyconfigs + additional:
                                 additional.append(ec)
                                 _log.debug("Added %s as dependency of %s" % (ec, entry))
                 else:
@@ -340,17 +403,20 @@ def resolve_dependencies(easyconfigs, modtool, retain_all_deps=False):
 
         elif not robot:
             # no use in continuing if robot is not enabled, dependencies won't be resolved anyway
-            irresolvable = [dep for x in easyconfigs for dep in x['dependencies']]
-            break
+            missing_deps = [dep for x in easyconfigs for dep in x['dependencies']]
+            if missing_deps:
+                raise_error_missing_deps(missing_deps, extra_msg="enable dependency resolution via --robot?")
 
-    if irresolvable:
-        _log.warning("Irresolvable dependencies (details): %s" % irresolvable)
-        irresolvable_mods_eb = [EasyBuildMNS().det_full_module_name(dep) for dep in irresolvable]
-        _log.warning("Irresolvable dependencies (EasyBuild module names): %s" % ', '.join(irresolvable_mods_eb))
-        irresolvable_mods = [ActiveMNS().det_full_module_name(dep) for dep in irresolvable]
-        raise EasyBuildError("Irresolvable dependencies encountered: %s", ', '.join(irresolvable_mods))
+    if totally_missing:
+        raise_error_missing_deps(totally_missing, extra_msg="no easyconfig file or existing module found")
 
-    _log.info("Dependency resolution complete, building as follows: %s" % ordered_ecs)
+    if missing_easyconfigs:
+        if raise_error_missing_ecs:
+            raise_error_missing_deps(missing_easyconfigs, extra_msg="no easyconfig file found in robot search path")
+        else:
+            _log.warning("No easyconfig files found for: %s", missing_easyconfigs)
+
+    _log.info("Dependency resolution complete, building as follows: %s", ordered_ecs)
     return ordered_ecs
 
 
@@ -359,6 +425,9 @@ def search_easyconfigs(query, short=False, filename_only=False, terse=False):
     search_path = build_option('robot_path')
     if not search_path:
         search_path = [os.getcwd()]
+    extra_search_paths = build_option('search_paths')
+    if extra_search_paths:
+        search_path.extend(extra_search_paths)
 
     ignore_dirs = build_option('ignore_dirs')
 
@@ -366,7 +435,7 @@ def search_easyconfigs(query, short=False, filename_only=False, terse=False):
     var_defs, _hits = search_file(search_path, query, short=short, ignore_dirs=ignore_dirs, terse=terse,
                                   silent=True, filename_only=False)
 
-     # filter out archived easyconfigs, these are handled separately
+    # filter out archived easyconfigs, these are handled separately
     hits, archived_hits = [], []
     for hit in _hits:
         if EASYCONFIGS_ARCHIVE_DIR in hit.split(os.path.sep):
@@ -402,4 +471,4 @@ def search_easyconfigs(query, short=False, filename_only=False, terse=False):
                 "Note: %d matching archived easyconfig(s) found, use --consider-archived-easyconfigs to see them" % cnt,
             ])
 
-    print '\n'.join(lines)
+    print('\n'.join(lines))
