@@ -1,5 +1,5 @@
 # #
-# Copyright 2009-2017 Ghent University
+# Copyright 2009-2019 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -8,7 +8,7 @@
 # Flemish Research Foundation (FWO) (http://www.fwo.be/en)
 # and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
 #
-# http://github.com/hpcugent/easybuild
+# https://github.com/easybuilders/easybuild
 #
 # EasyBuild is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -38,14 +38,16 @@ import os
 import re
 import signal
 import subprocess
+import sys
 import tempfile
 import time
-
+from datetime import datetime
 from vsc.utils import fancylogger
 
 from easybuild.tools.asyncprocess import PIPE, STDOUT, Popen, recv_some, send_all
-from easybuild.tools.config import build_option
-from easybuild.tools.build_log import EasyBuildError, dry_run_msg
+from easybuild.tools.build_log import EasyBuildError, dry_run_msg, print_msg, time_str_since
+from easybuild.tools.config import ERROR, IGNORE, WARN, build_option
+from easybuild.tools.utilities import trace_msg
 
 
 _log = fancylogger.getLogger('run', fname=False)
@@ -53,11 +55,6 @@ _log = fancylogger.getLogger('run', fname=False)
 
 errors_found_in_log = 0
 
-# constants for strictness levels
-IGNORE = 'ignore'
-WARN = 'warn'
-ERROR = 'error'
-
 # default strictness level
 strictness = WARN
 
@@ -80,26 +77,28 @@ def run_cmd_cache(func):
     @functools.wraps(func)
     def cache_aware_func(cmd, *args, **kwargs):
         """Retrieve cached result of selected commands, or run specified and collect & cache result."""
-
-        # fetch from cache if available, cache it if it's not
-        if cmd in cache:
-            _log.debug("Using cached value for command '%s': %s", cmd, cache[cmd])
-            return cache[cmd]
+        # cache key is combination of command and input provided via stdin
+        key = (cmd, kwargs.get('inp', None))
+        # fetch from cache if available, cache it if it's not, but only on cmd strings
+        if isinstance(cmd, basestring) and key in cache:
+            _log.debug("Using cached value for command '%s': %s", cmd, cache[key])
+            return cache[key]
         else:
             res = func(cmd, *args, **kwargs)
             if cmd in CACHED_COMMANDS:
-                cache[cmd] = res
+                cache[key] = res
             return res
 
-    # expose clear method of cache to wrapped function
+    # expose clear/update methods of cache to wrapped function
     cache_aware_func.clear_cache = cache.clear
+    cache_aware_func.update_cache = cache.update
 
     return cache_aware_func
 
 
 @run_cmd_cache
 def run_cmd(cmd, log_ok=True, log_all=False, simple=False, inp=None, regexp=True, log_output=False, path=None,
-            force_in_dry_run=False, verbose=True):
+            force_in_dry_run=False, verbose=True, shell=True, trace=True, stream_output=None):
     """
     Run specified command (in a subshell)
     :param cmd: command to run
@@ -112,15 +111,52 @@ def run_cmd(cmd, log_ok=True, log_all=False, simple=False, inp=None, regexp=True
     :param path: path to execute the command in; current working directory is used if unspecified
     :param force_in_dry_run: force running the command during dry run
     :param verbose: include message on running the command in dry run output
+    :param shell: allow commands to not run in a shell (especially useful for cmd lists)
+    :param trace: print command being executed as part of trace output
+    :param stream_output: enable streaming command output to stdout
     """
     cwd = os.getcwd()
 
+    if isinstance(cmd, basestring):
+        cmd_msg = cmd.strip()
+    elif isinstance(cmd, list):
+        cmd_msg = ' '.join(cmd)
+    else:
+        raise EasyBuildError("Unknown command type ('%s'): %s", type(cmd), cmd)
+
+    if log_output or (trace and build_option('trace')):
+        # collect output of running command in temporary log file, if desired
+        fd, cmd_log_fn = tempfile.mkstemp(suffix='.log', prefix='easybuild-run_cmd-')
+        os.close(fd)
+        try:
+            cmd_log = open(cmd_log_fn, 'w')
+        except IOError as err:
+            raise EasyBuildError("Failed to open temporary log file for output of command: %s", err)
+        _log.debug('run_cmd: Output of "%s" will be logged to %s' % (cmd, cmd_log_fn))
+    else:
+        cmd_log_fn, cmd_log = None, None
+
+    # auto-enable streaming of command output under --logtostdout/-l, unless it was disabled explicitely
+    if stream_output is None and build_option('logtostdout'):
+        _log.info("Auto-enabling streaming output of '%s' command because logging to stdout is enabled", cmd_msg)
+        stream_output = True
+
+    if stream_output:
+        print_msg("(streaming) output for command '%s':" % cmd_msg)
+
+    start_time = datetime.now()
+    if trace:
+        trace_txt = "running command:\n"
+        trace_txt += "\t[started at: %s]\n" % start_time.strftime('%Y-%m-%d %H:%M:%S')
+        trace_txt += "\t[output logged in %s]\n" % cmd_log_fn
+        trace_msg(trace_txt + '\t' + cmd_msg)
+
     # early exit in 'dry run' mode, after printing the command that would be run (unless running the command is forced)
     if not force_in_dry_run and build_option('extended_dry_run'):
         if path is None:
             path = cwd
         if verbose:
-            dry_run_msg("  running command \"%s\"" % cmd, silent=build_option('silent'))
+            dry_run_msg("  running command \"%s\"" % cmd_msg, silent=build_option('silent'))
             dry_run_msg("  (in %s)" % path, silent=build_option('silent'))
 
         # make sure we get the type of the return value right
@@ -135,62 +171,76 @@ def run_cmd(cmd, log_ok=True, log_all=False, simple=False, inp=None, regexp=True
             os.chdir(path)
 
         _log.debug("run_cmd: running cmd %s (in %s)" % (cmd, os.getcwd()))
-    except OSError, err:
+    except OSError as err:
         _log.warning("Failed to change to %s: %s" % (path, err))
-        _log.info("running cmd %s in non-existing directory, might fail!" % cmd)
+        _log.info("running cmd %s in non-existing directory, might fail!", cmd)
 
-    # # Log command output
-    if log_output:
-        runLog = tempfile.NamedTemporaryFile(suffix='.log', prefix='easybuild-run_cmd-')
-        _log.debug('run_cmd: Command output will be logged to %s' % runLog.name)
-        runLog.write(cmd + "\n\n")
-    else:
-        runLog = None
+    if cmd_log:
+        cmd_log.write("# output for command: %s\n\n" % cmd_msg)
 
-    readSize = 1024 * 8
+    exec_cmd = "/bin/bash"
 
+    if not shell:
+        if isinstance(cmd, list):
+            exec_cmd = None
+            cmd.insert(0, '/usr/bin/env')
+        elif isinstance(cmd, basestring):
+            cmd = '/usr/bin/env %s' % cmd
+        else:
+            raise EasyBuildError("Don't know how to prefix with /usr/bin/env for commands of type %s", type(cmd))
+
+    _log.info('running cmd: %s ' % cmd)
     try:
-        p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
-                             stdin=subprocess.PIPE, close_fds=True, executable="/bin/bash")
-    except OSError, err:
+        proc = subprocess.Popen(cmd, shell=shell, stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
+                                stdin=subprocess.PIPE, close_fds=True, executable=exec_cmd)
+    except OSError as err:
         raise EasyBuildError("run_cmd init cmd %s failed:%s", cmd, err)
     if inp:
-        p.stdin.write(inp)
-    p.stdin.close()
+        proc.stdin.write(inp)
+    proc.stdin.close()
 
-    ec = p.poll()
+    # use small read size when streaming output, to make it stream more fluently
+    # read size should not be too small though, to avoid too much overhead
+    if stream_output:
+        read_size = 128
+    else:
+        read_size = 1024 * 8
+
+    ec = proc.poll()
     stdouterr = ''
     while ec is None:
         # need to read from time to time.
         # - otherwise the stdout/stderr buffer gets filled and it all stops working
-        output = p.stdout.read(readSize)
-        if runLog:
-            runLog.write(output)
+        output = proc.stdout.read(read_size)
+        if cmd_log:
+            cmd_log.write(output)
+        if stream_output:
+            sys.stdout.write(output)
         stdouterr += output
-        ec = p.poll()
+        ec = proc.poll()
 
     # read remaining data (all of it)
-    output = p.stdout.read()
-    if runLog:
-        runLog.write(output)
+    output = proc.stdout.read()
+    if cmd_log:
+        cmd_log.write(output)
+        cmd_log.close()
+    if stream_output:
+        sys.stdout.write(output)
     stdouterr += output
 
-    # not needed anymore. subprocess does this correct?
-    # ec=os.WEXITSTATUS(ec)
-
-    # # Command log output
-    if log_output:
-        runLog.close()
+    if trace:
+        trace_msg("command completed: exit %s, ran in %s" % (ec, time_str_since(start_time)))
 
     try:
         os.chdir(cwd)
-    except OSError, err:
+    except OSError as err:
         raise EasyBuildError("Failed to return to %s after executing command: %s", cwd, err)
 
     return parse_cmd_output(cmd, stdouterr, ec, simple, log_all, log_ok, regexp)
 
 
-def run_cmd_qa(cmd, qa, no_qa=None, log_ok=True, log_all=False, simple=False, regexp=True, std_qa=None, path=None, maxhits=50):
+def run_cmd_qa(cmd, qa, no_qa=None, log_ok=True, log_all=False, simple=False, regexp=True, std_qa=None, path=None,
+               maxhits=50, trace=True):
     """
     Run specified interactive command (in a subshell)
     :param cmd: command to run
@@ -202,9 +252,30 @@ def run_cmd_qa(cmd, qa, no_qa=None, log_ok=True, log_all=False, simple=False, re
     :param regex: regex used to check the output for errors; if True it will use the default (see parse_log_for_error)
     :param std_qa: dictionary which maps question regex patterns to answers
     :param path: path to execute the command is; current working directory is used if unspecified
+    :param maxhits: maximum number of cycles (seconds) without being able to find a known question
+    :param trace: print command being executed as part of trace output
     """
     cwd = os.getcwd()
 
+    if log_all or (trace and build_option('trace')):
+        # collect output of running command in temporary log file, if desired
+        fd, cmd_log_fn = tempfile.mkstemp(suffix='.log', prefix='easybuild-run_cmd_qa-')
+        os.close(fd)
+        try:
+            cmd_log = open(cmd_log_fn, 'w')
+        except IOError as err:
+            raise EasyBuildError("Failed to open temporary log file for output of interactive command: %s", err)
+        _log.debug('run_cmd_qa: Output of "%s" will be logged to %s' % (cmd, cmd_log_fn))
+    else:
+        cmd_log_fn, cmd_log = None, None
+
+    start_time = datetime.now()
+    if trace:
+        trace_txt = "running interactive command:\n"
+        trace_txt += "\t[started at: %s]\n" % start_time.strftime('%Y-%m-%d %H:%M:%S')
+        trace_txt += "\t[output logged in %s]\n" % cmd_log_fn
+        trace_msg(trace_txt + '\t' + cmd.strip())
+
     # early exit in 'dry run' mode, after printing the command that would be run
     if build_option('extended_dry_run'):
         if path is None:
@@ -222,7 +293,7 @@ def run_cmd_qa(cmd, qa, no_qa=None, log_ok=True, log_all=False, simple=False, re
             os.chdir(path)
 
         _log.debug("run_cmd_qa: running cmd %s (in %s)" % (cmd, os.getcwd()))
-    except OSError, err:
+    except OSError as err:
         _log.warning("Failed to change to %s: %s" % (path, err))
         _log.info("running cmd %s in non-existing directory, might fail!" % cmd)
 
@@ -261,23 +332,23 @@ def run_cmd_qa(cmd, qa, no_qa=None, log_ok=True, log_all=False, simple=False, re
         # list is manipulated when answering matching question, so return a copy
         return answers[:]
 
-    newQA = {}
-    _log.debug("newQA: ")
+    new_qa = {}
+    _log.debug("new_qa: ")
     for question, answers in qa.items():
         answers = check_answers_list(answers)
         (answers, regQ) = process_QA(question, answers)
-        newQA[regQ] = answers
-        _log.debug("newqa[%s]: %s" % (regQ.pattern, newQA[regQ]))
+        new_qa[regQ] = answers
+        _log.debug("new_qa[%s]: %s" % (regQ.pattern, new_qa[regQ]))
 
-    newstdQA = {}
+    new_std_qa = {}
     if std_qa:
         for question, answers in std_qa.items():
             regQ = re.compile(r"" + question + r"[\s\n]*$")
             answers = check_answers_list(answers)
             for i in [idx for idx, a in enumerate(answers) if not a.endswith('\n')]:
                 answers[i] += '\n'
-            newstdQA[regQ] = answers
-            _log.debug("newstdQA[%s]: %s" % (regQ.pattern, newstdQA[regQ]))
+            new_std_qa[regQ] = answers
+            _log.debug("new_std_qa[%s]: %s" % (regQ.pattern, new_std_qa[regQ]))
 
     new_no_qa = []
     if no_qa:
@@ -290,93 +361,86 @@ def run_cmd_qa(cmd, qa, no_qa=None, log_ok=True, log_all=False, simple=False, re
     # - this needs asynchronous stdout
 
     # # Log command output
-    if log_all:
-        try:
-            runLog = tempfile.NamedTemporaryFile(suffix='.log', prefix='easybuild-cmdqa-')
-            _log.debug('run_cmd_qa: Command output will be logged to %s' % runLog.name)
-            runLog.write(cmd + "\n\n")
-        except IOError, err:
-            raise EasyBuildError("Opening log file for Q&A failed: %s", err)
-    else:
-        runLog = None
+    if cmd_log:
+        cmd_log.write("# output for interactive command: %s\n\n" % cmd)
 
     try:
         p = Popen(cmd, shell=True, stdout=PIPE, stderr=STDOUT, stdin=PIPE, close_fds=True, executable="/bin/bash")
-    except OSError, err:
+    except OSError as err:
         raise EasyBuildError("run_cmd_qa init cmd %s failed:%s", cmd, err)
 
     ec = p.poll()
-    stdoutErr = ''
-    oldLenOut = -1
-    hitCount = 0
+    stdout_err = ''
+    old_len_out = -1
+    hit_count = 0
 
     while ec is None:
         # need to read from time to time.
         # - otherwise the stdout/stderr buffer gets filled and it all stops working
         try:
-            tmpOut = recv_some(p)
-            if runLog:
-                runLog.write(tmpOut)
-            stdoutErr += tmpOut
+            out = recv_some(p)
+            if cmd_log:
+                cmd_log.write(out)
+            stdout_err += out
         # recv_some may throw Exception
-        except (IOError, Exception), err:
-            _log.debug("run_cmd_qa cmd %s: read failed: %s" % (cmd, err))
-            tmpOut = None
+        except (IOError, Exception) as err:
+            _log.debug("run_cmd_qa cmd %s: read failed: %s", cmd, err)
+            out = None
 
         hit = False
-        for question, answers in newQA.items():
-            res = question.search(stdoutErr)
-            if tmpOut and res:
+        for question, answers in new_qa.items():
+            res = question.search(stdout_err)
+            if out and res:
                 fa = answers[0] % res.groupdict()
                 # cycle through list of answers
                 last_answer = answers.pop(0)
                 answers.append(last_answer)
-                _log.debug("List of answers for question %s after cycling: %s" % (question.pattern, answers))
+                _log.debug("List of answers for question %s after cycling: %s", question.pattern, answers)
 
-                _log.debug("run_cmd_qa answer %s question %s out %s" % (fa, question.pattern, stdoutErr[-50:]))
+                _log.debug("run_cmd_qa answer %s question %s out %s", fa, question.pattern, stdout_err[-50:])
                 send_all(p, fa)
                 hit = True
                 break
         if not hit:
-            for question, answers in newstdQA.items():
-                res = question.search(stdoutErr)
-                if tmpOut and res:
+            for question, answers in new_std_qa.items():
+                res = question.search(stdout_err)
+                if out and res:
                     fa = answers[0] % res.groupdict()
                     # cycle through list of answers
                     last_answer = answers.pop(0)
                     answers.append(last_answer)
-                    _log.debug("List of answers for question %s after cycling: %s" % (question.pattern, answers))
+                    _log.debug("List of answers for question %s after cycling: %s", question.pattern, answers)
 
-                    _log.debug("run_cmd_qa answer %s std question %s out %s" % (fa, question.pattern, stdoutErr[-50:]))
+                    _log.debug("run_cmd_qa answer %s std question %s out %s", fa, question.pattern, stdout_err[-50:])
                     send_all(p, fa)
                     hit = True
                     break
             if not hit:
-                if len(stdoutErr) > oldLenOut:
-                    oldLenOut = len(stdoutErr)
+                if len(stdout_err) > old_len_out:
+                    old_len_out = len(stdout_err)
                 else:
                     noqa = False
                     for r in new_no_qa:
-                        if r.search(stdoutErr):
-                            _log.debug("runqanda: noQandA found for out %s" % stdoutErr[-50:])
+                        if r.search(stdout_err):
+                            _log.debug("runqanda: noQandA found for out %s", stdout_err[-50:])
                             noqa = True
                     if not noqa:
-                        hitCount += 1
+                        hit_count += 1
             else:
-                hitCount = 0
+                hit_count = 0
         else:
-            hitCount = 0
+            hit_count = 0
 
-        if hitCount > maxhits:
+        if hit_count > maxhits:
             # explicitly kill the child process before exiting
             try:
                 os.killpg(p.pid, signal.SIGKILL)
                 os.kill(p.pid, signal.SIGKILL)
-            except OSError, err:
-                _log.debug("run_cmd_qa exception caught when killing child process: %s" % err)
-            _log.debug("run_cmd_qa: full stdouterr: %s" % stdoutErr)
+            except OSError as err:
+                _log.debug("run_cmd_qa exception caught when killing child process: %s", err)
+            _log.debug("run_cmd_qa: full stdouterr: %s", stdout_err)
             raise EasyBuildError("run_cmd_qa: cmd %s : Max nohits %s reached: end of output %s",
-                                 cmd, maxhits, stdoutErr[-500:])
+                                 cmd, maxhits, stdout_err[-500:])
 
         # the sleep below is required to avoid exiting on unknown 'questions' too early (see above)
         time.sleep(1)
@@ -385,22 +449,23 @@ def run_cmd_qa(cmd, qa, no_qa=None, log_ok=True, log_all=False, simple=False, re
     # Process stopped. Read all remaining data
     try:
         if p.stdout:
-            readTxt = p.stdout.read()
-            stdoutErr += readTxt
-            if runLog:
-                runLog.write(readTxt)
-    except IOError, err:
-        _log.debug("runqanda cmd %s: remaining data read failed: %s" % (cmd, err))
+            out = p.stdout.read()
+            stdout_err += out
+            if cmd_log:
+                cmd_log.write(out)
+                cmd_log.close()
+    except IOError as err:
+        _log.debug("runqanda cmd %s: remaining data read failed: %s", cmd, err)
 
-    # Not needed anymore. Subprocess does this correct?
-    # ec=os.WEXITSTATUS(ec)
+    if trace:
+        trace_msg("interactive command completed: exit %s, ran in %s" % (ec, time_str_since(start_time)))
 
     try:
         os.chdir(cwd)
-    except OSError, err:
+    except OSError as err:
         raise EasyBuildError("Failed to return to %s after executing command: %s", cwd, err)
 
-    return parse_cmd_output(cmd, stdoutErr, ec, simple, log_all, log_ok, regexp)
+    return parse_cmd_output(cmd, stdout_err, ec, simple, log_all, log_ok, regexp)
 
 
 def parse_cmd_output(cmd, stdouterr, ec, simple, log_all, log_ok, regexp):
@@ -430,17 +495,17 @@ def parse_cmd_output(cmd, stdouterr, ec, simple, log_all, log_ok, regexp):
     if not regexp:
         use_regexp = False
 
-    _log.debug('cmd "%s" exited with exitcode %s and output:\n%s' % (cmd, ec, stdouterr))
-
     if ec and (log_all or log_ok):
         # We don't want to error if the user doesn't care
         if check_ec:
-            raise EasyBuildError('cmd "%s" exited with exitcode %s and output:\n%s', cmd, ec, stdouterr)
+            raise EasyBuildError('cmd "%s" exited with exit code %s and output:\n%s', cmd, ec, stdouterr)
         else:
-            _log.warn('cmd "%s" exited with exitcode %s and output:\n%s' % (cmd, ec, stdouterr))
+            _log.warn('cmd "%s" exited with exit code %s and output:\n%s' % (cmd, ec, stdouterr))
     elif not ec:
         if log_all:
-            _log.info('cmd "%s" exited with exitcode %s and output:\n%s' % (cmd, ec, stdouterr))
+            _log.info('cmd "%s" exited with exit code %s and output:\n%s' % (cmd, ec, stdouterr))
+        else:
+            _log.debug('cmd "%s" exited with exit code %s and output:\n%s' % (cmd, ec, stdouterr))
 
     # parse the stdout/stderr for errors when strictness dictates this or when regexp is passed in
     if use_regexp or regexp:
@@ -496,5 +561,3 @@ def parse_log_for_error(txt, regExp=None, stdout=True, msg=None):
                   (regExp, '\n'.join([x[0] for x in res])))
 
     return res
-
-
