@@ -1,5 +1,5 @@
 ##
-# Copyright 2011-2017 Ghent University
+# Copyright 2011-2019 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -8,7 +8,7 @@
 # Flemish Research Foundation (FWO) (http://www.fwo.be/en)
 # and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
 #
-# http://github.com/hpcugent/easybuild
+# https://github.com/easybuilders/easybuild
 #
 # EasyBuild is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -118,6 +118,10 @@ ARM_CORTEX_IDS = {
     '0xd09': 'Cortex-A73',
 }
 
+# OS package handler name constants
+RPM = 'rpm'
+DPKG = 'dpkg'
+
 
 class SystemToolsException(Exception):
     """raised when systemtools fails"""
@@ -135,7 +139,7 @@ def get_avail_core_count():
         core_cnt = int(sum(sched_getaffinity().cpus))
     else:
         # BSD-type systems
-        out, _ = run_cmd('sysctl -n hw.ncpu', force_in_dry_run=True)
+        out, _ = run_cmd('sysctl -n hw.ncpu', force_in_dry_run=True, trace=False, stream_output=False)
         try:
             if int(out) > 0:
                 core_cnt = int(out)
@@ -172,7 +176,7 @@ def get_total_memory():
     elif os_type == DARWIN:
         cmd = "sysctl -n hw.memsize"
         _log.debug("Trying to determine total memory size on Darwin via cmd '%s'", cmd)
-        out, ec = run_cmd(cmd, force_in_dry_run=True)
+        out, ec = run_cmd(cmd, force_in_dry_run=True, trace=False, stream_output=False)
         if ec == 0:
             memtotal = int(out.strip()) / (1024**2)
 
@@ -248,7 +252,7 @@ def get_cpu_vendor():
 
     elif os_type == DARWIN:
         cmd = "sysctl -n machdep.cpu.vendor"
-        out, ec = run_cmd(cmd, force_in_dry_run=True)
+        out, ec = run_cmd(cmd, force_in_dry_run=True, trace=False, stream_output=False)
         out = out.strip()
         if ec == 0 and out in VENDOR_IDS:
             vendor = VENDOR_IDS[out]
@@ -332,7 +336,7 @@ def get_cpu_model():
 
     elif os_type == DARWIN:
         cmd = "sysctl -n machdep.cpu.brand_string"
-        out, ec = run_cmd(cmd, force_in_dry_run=True)
+        out, ec = run_cmd(cmd, force_in_dry_run=True, trace=False, stream_output=False)
         if ec == 0:
             model = out.strip()
             _log.debug("Determined CPU model on Darwin using cmd '%s': %s" % (cmd, model))
@@ -377,7 +381,7 @@ def get_cpu_speed():
     elif os_type == DARWIN:
         cmd = "sysctl -n hw.cpufrequency_max"
         _log.debug("Trying to determine CPU frequency on Darwin via cmd '%s'" % cmd)
-        out, ec = run_cmd(cmd, force_in_dry_run=True)
+        out, ec = run_cmd(cmd, force_in_dry_run=True, trace=False, stream_output=False)
         if ec == 0:
             # returns clock frequency in cycles/sec, but we want MHz
             cpu_freq = float(out.strip()) / (1000 ** 2)
@@ -423,7 +427,7 @@ def get_cpu_features():
         for feature_set in ['extfeatures', 'features', 'leaf7_features']:
             cmd = "sysctl -n machdep.cpu.%s" % feature_set
             _log.debug("Trying to determine CPU features on Darwin via cmd '%s'", cmd)
-            out, ec = run_cmd(cmd, force_in_dry_run=True)
+            out, ec = run_cmd(cmd, force_in_dry_run=True, trace=False, stream_output=False)
             if ec == 0:
                 cpu_feat.extend(out.strip().lower().split())
 
@@ -563,24 +567,41 @@ def check_os_dependency(dep):
     # - uses rpm -q and dpkg -s --> can be run as non-root!!
     # - fallback on which
     # - should be extended to files later?
-    found = None
+    found = False
     cmd = None
-    if which('rpm'):
-        cmd = "rpm -q %s" % dep
-        found = run_cmd(cmd, simple=True, log_all=False, log_ok=False, force_in_dry_run=True)
+    os_to_pkg_cmd_map = {
+        'centos': RPM,
+        'debian': DPKG,
+        'redhat': RPM,
+        'ubuntu': DPKG,
+    }
+    pkg_cmd_flag = {
+        DPKG: '-s',
+        RPM: '-q',
+    }
+    os_name = get_os_name()
+    if os_name in os_to_pkg_cmd_map:
+        pkg_cmds = [os_to_pkg_cmd_map[os_name]]
+    else:
+        pkg_cmds = [RPM, DPKG]
 
-    if not found and which('dpkg'):
-        cmd = "dpkg -s %s" % dep
-        found = run_cmd(cmd, simple=True, log_all=False, log_ok=False, force_in_dry_run=True)
+    for pkg_cmd in pkg_cmds:
+        if which(pkg_cmd):
+            cmd = ' '.join([pkg_cmd, pkg_cmd_flag.get(pkg_cmd), dep])
+            found = run_cmd(cmd, simple=True, log_all=False, log_ok=False,
+                            force_in_dry_run=True, trace=False, stream_output=False)
+            if found:
+                break
 
-    if cmd is None:
+    if not found:
         # fallback for when os-dependency is a binary/library
         found = which(dep)
 
         # try locate if it's available
         if not found and which('locate'):
             cmd = 'locate --regexp "/%s$"' % dep
-            found = run_cmd(cmd, simple=True, log_all=False, log_ok=False, force_in_dry_run=True)
+            found = run_cmd(cmd, simple=True, log_all=False, log_ok=False, force_in_dry_run=True, trace=False,
+                            stream_output=False)
 
     return found
 
@@ -590,7 +611,8 @@ def get_tool_version(tool, version_option='--version'):
     Get output of running version option for specific command line tool.
     Output is returned as a single-line string (newlines are replaced by '; ').
     """
-    out, ec = run_cmd(' '.join([tool, version_option]), simple=False, log_ok=False, force_in_dry_run=True)
+    out, ec = run_cmd(' '.join([tool, version_option]), simple=False, log_ok=False, force_in_dry_run=True,
+                      trace=False, stream_output=False)
     if ec:
         _log.warning("Failed to determine version of %s using '%s %s': %s" % (tool, tool, version_option, out))
         return UNKNOWN
@@ -602,7 +624,8 @@ def get_gcc_version():
     """
     Process `gcc --version` and return the GCC version.
     """
-    out, ec = run_cmd('gcc --version', simple=False, log_ok=False, force_in_dry_run=True, verbose=False)
+    out, ec = run_cmd('gcc --version', simple=False, log_ok=False, force_in_dry_run=True, verbose=False, trace=False,
+                      stream_output=False)
     res = None
     if ec:
         _log.warning("Failed to determine the version of GCC: %s", out)
@@ -629,6 +652,7 @@ def get_glibc_version():
     """
     Find the version of glibc used on this system
     """
+    glibc_ver = UNKNOWN
     os_type = get_os_type()
 
     if os_type == LINUX:
@@ -637,16 +661,16 @@ def get_glibc_version():
         res = glibc_ver_regex.search(glibc_ver_str)
 
         if res is not None:
-            glibc_version = res.group(1)
-            _log.debug("Found glibc version %s" % glibc_version)
-            return glibc_version
+            glibc_ver = res.group(1)
+            _log.debug("Found glibc version %s" % glibc_ver)
         else:
-            raise EasyBuildError("Failed to determine glibc version from '%s' using pattern '%s'.",
-                                 glibc_ver_str, glibc_ver_regex.pattern)
+            _log.warning("Failed to determine glibc version from '%s' using pattern '%s'.",
+                         glibc_ver_str, glibc_ver_regex.pattern)
     else:
         # no glibc on OS X standard
         _log.debug("No glibc on a non-Linux system, so can't determine version.")
-        return UNKNOWN
+
+    return glibc_ver
 
 
 def get_system_info():
@@ -675,13 +699,13 @@ def use_group(group_name):
     """Use group with specified name."""
     try:
         group_id = grp.getgrnam(group_name).gr_gid
-    except KeyError, err:
+    except KeyError as err:
         raise EasyBuildError("Failed to get group ID for '%s', group does not exist (err: %s)", group_name, err)
 
     group = (group_name, group_id)
     try:
         os.setgid(group_id)
-    except OSError, err:
+    except OSError as err:
         err_msg = "Failed to use group %s: %s; " % (group, err)
         user = pwd.getpwuid(os.getuid()).pw_name
         grp_members = grp.getgrgid(group_id).gr_mem
@@ -704,12 +728,12 @@ def det_parallelism(par=None, maxpar=None):
         if not isinstance(par, int):
             try:
                 par = int(par)
-            except ValueError, err:
+            except ValueError as err:
                 raise EasyBuildError("Specified level of parallelism '%s' is not an integer value: %s", par, err)
     else:
         par = get_avail_core_count()
         # check ulimit -u
-        out, ec = run_cmd('ulimit -u', force_in_dry_run=True)
+        out, ec = run_cmd('ulimit -u', force_in_dry_run=True, trace=False, stream_output=False)
         try:
             if out.startswith("unlimited"):
                 out = 2 ** 32 - 1
@@ -719,7 +743,7 @@ def det_parallelism(par=None, maxpar=None):
             if par_guess < par:
                 par = par_guess
                 _log.info("Limit parallel builds to %s because max user processes is %s" % (par, out))
-        except ValueError, err:
+        except ValueError as err:
             raise EasyBuildError("Failed to determine max user processes (%s, %s): %s", ec, out, err)
 
     if maxpar is not None and maxpar < par:
