@@ -1,5 +1,5 @@
 ##
-# Copyright 2012-2017 Ghent University
+# Copyright 2012-2019 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -8,7 +8,7 @@
 # Flemish Research Foundation (FWO) (http://www.fwo.be/en)
 # and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
 #
-# http://github.com/hpcugent/easybuild
+# https://github.com/easybuilders/easybuild
 #
 # EasyBuild is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -56,12 +56,15 @@ class Fft(Toolchain):
         self.variables.add_begin_end_linkerflags(fft_libs, toggle_startstopgroup=self.FFT_LIB_GROUP,
                                                  toggle_staticdynamic=self.FFT_LIB_STATIC)
 
-        ## multi-threaded
+        # multi-threaded
         if self.FFT_LIB_MT is None:
-            ## reuse FFT variables
+            # reuse FFT variables
             self.variables.join('LIBFFT_MT', 'LIBFFT')
         else:
-            self.variables.nappend('LIBFFT_MT', self.FFT_LIB_MT)
+            fft_mt_libs = self.variables.nappend('LIBFFT_MT', self.FFT_LIB_MT)
+            self.variables.add_begin_end_linkerflags(fft_mt_libs, toggle_startstopgroup=self.FFT_LIB_GROUP,
+                                                     toggle_staticdynamic=self.FFT_LIB_STATIC)
+
             if getattr(self, 'LIB_MULTITHREAD', None) is not None:
                 self.variables.nappend('LIBFFT_MT', self.LIB_MULTITHREAD)
 
@@ -76,9 +79,8 @@ class Fft(Toolchain):
 
     def set_variables(self):
         """Set the variables"""
-        ## TODO is link order fully preserved with this order ?
         self._set_fft_variables()
 
-        self.log.debug('set_variables: FFT variables %s' % self.variables)
+        self.log.devel('set_variables: FFT variables %s', self.variables)
 
         super(Fft, self).set_variables()
