@@ -1,5 +1,5 @@
 # #
-# Copyright 2012-2017 Ghent University
+# Copyright 2012-2019 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -8,7 +8,7 @@
 # Flemish Research Foundation (FWO) (http://www.fwo.be/en)
 # and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
 #
-# http://github.com/hpcugent/easybuild
+# https://github.com/easybuilders/easybuild
 #
 # EasyBuild is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@ Toolchain mpi module. Contains all MPI related classes
 """
 import os
 import tempfile
+from distutils.version import LooseVersion
 
 import easybuild.tools.environment as env
 import easybuild.tools.toolchain as toolchain
@@ -53,8 +54,8 @@ class Mpi(Toolchain):
 
     MPI_UNIQUE_OPTS = None
     MPI_SHARED_OPTS = {
-                       'usempi': (False, "Use MPI compiler as default compiler"),  # also FFTW
-                       }
+        'usempi': (False, "Use MPI compiler as default compiler"),  # also FFTW
+    }
 
     MPI_UNIQUE_OPTION_MAP = None
     MPI_SHARED_OPTION_MAP = {
@@ -87,7 +88,7 @@ class Mpi(Toolchain):
 
         self.options.add_options(self.MPI_UNIQUE_OPTS, self.MPI_UNIQUE_OPTION_MAP)
 
-        self.log.debug('_set_mpi_options: all current options %s' % self.options)
+        self.log.devel('_set_mpi_options: all current options %s', self.options)
 
 
     def set_variables(self):
@@ -95,14 +96,14 @@ class Mpi(Toolchain):
         self._set_mpi_compiler_variables()
         self._set_mpi_variables()
 
-        self.log.debug('set_variables: compiler variables %s' % self.variables)
+        self.log.devel('set_variables: compiler variables %s', self.variables)
         super(Mpi, self).set_variables()
 
     def _set_mpi_compiler_variables(self):
         """Set the MPI compiler variables"""
         is32bit = self.options.get('32bit', None)
         if is32bit:
-            self.log.debug("_set_compiler_variables: 32bit set: changing compiler definitions")
+            self.log.debug("_set_mpi_compiler_variables: 32bit set: changing compiler definitions")
 
         for var_tuple in COMPILER_VARIABLES:
             c_var = var_tuple[0]  # [1] is the description
@@ -126,16 +127,16 @@ class Mpi(Toolchain):
 
             if self.options.get('usempi', None):
                 var_seq = SEQ_COMPILER_TEMPLATE % {'c_var': c_var}
-                self.log.debug('_set_mpi_compiler_variables: usempi set: defining %s as %s' % (var_seq, self.variables[c_var]))
+                self.log.debug("usempi set: defining %s as %s", var_seq, self.variables[c_var])
                 self.variables[var_seq] = self.variables[c_var]
-                self.log.debug("_set_mpi_compiler_variables: usempi set: switching %s value %s for %s value %s" %
-                               (c_var, self.variables[c_var], var, self.variables[var]))
+                self.log.debug("usempi set: switching %s value %s for %s value %s",
+                               c_var, self.variables[c_var], var, self.variables[var])
                 self.variables[c_var] = self.variables[var]
 
 
         if self.options.get('cciscxx', None):
-            self.log.debug("_set_mpi_compiler_variables: cciscxx set: switching MPICXX %s for MPICC value %s" %
-                           (self.variables['MPICXX'], self.variables['MPICC']))
+            self.log.debug("_set_mpi_compiler_variables: cciscxx set: switching MPICXX %s for MPICC value %s",
+                           self.variables['MPICXX'], self.variables['MPICC'])
             self.variables['MPICXX'] = self.variables['MPICC']
             if self.options.get('usempi', None):
                 # possibly/likely changed
@@ -165,8 +166,6 @@ class Mpi(Toolchain):
         else:
             raise EasyBuildError("mpi_family: MPI_FAMILY is undefined.")
 
-    # FIXME: deprecate this function, use mympirun instead
-    # this requires that either mympirun is packaged together with EasyBuild, or that vsc-tools is a dependency of EasyBuild
     def mpi_cmd_for(self, cmd, nr_ranks):
         """Construct an MPI command for the given command and number of ranks."""
 
@@ -183,60 +182,58 @@ class Mpi(Toolchain):
             # different known mpirun commands
             mpirun_n_cmd = "mpirun -n %(nr_ranks)d %(cmd)s"
             mpi_cmds = {
-                toolchain.OPENMPI: mpirun_n_cmd,  # @UndefinedVariable
-                toolchain.QLOGICMPI: "mpirun -H localhost -np %(nr_ranks)d %(cmd)s",  # @UndefinedVariable
-                toolchain.INTELMPI: "mpirun %(mpdbf)s %(nodesfile)s -np %(nr_ranks)d %(cmd)s",  # @UndefinedVariable
-                toolchain.MVAPICH2: mpirun_n_cmd,  # @UndefinedVariable
-                toolchain.MPICH: mpirun_n_cmd,  # @UndefinedVariable
-                toolchain.MPICH2: mpirun_n_cmd,  # @UndefinedVariable
+                toolchain.OPENMPI: mpirun_n_cmd,
+                toolchain.QLOGICMPI: "mpirun -H localhost -np %(nr_ranks)d %(cmd)s",
+                toolchain.INTELMPI: mpirun_n_cmd,
+                toolchain.MVAPICH2: mpirun_n_cmd,
+                toolchain.MPICH: mpirun_n_cmd,
+                toolchain.MPICH2: mpirun_n_cmd,
             }
 
         mpi_family = self.mpi_family()
 
         # Intel MPI mpirun needs more work
-        if mpi_cmd_template is None and mpi_family == toolchain.INTELMPI:  # @UndefinedVariable
-
-            # set temporary dir for mdp
-            # note: this needs to be kept *short*, to avoid mpirun failing with "socket.error: AF_UNIX path too long"
-            # exact limit is unknown, but ~20 characters seems to be OK
-            env.setvar('I_MPI_MPD_TMPDIR', tempfile.gettempdir())
-            mpd_tmpdir = os.environ['I_MPI_MPD_TMPDIR']
-            if len(mpd_tmpdir) > 20:
-                self.log.warning("$I_MPI_MPD_TMPDIR should be (very) short to avoid problems: %s" % mpd_tmpdir)
-
-            # temporary location for mpdboot and nodes files
-            tmpdir = tempfile.mkdtemp(prefix='mpi_cmd_for-')
-
-            # set PBS_ENVIRONMENT, so that --file option for mpdboot isn't stripped away
-            env.setvar('PBS_ENVIRONMENT', "PBS_BATCH_MPI")
-
-            # make sure we're always using mpd as process manager
-            # only required for/picked up by Intel MPI v4.1 or higher, no harm done for others
-            env.setvar('I_MPI_PROCESS_MANAGER', 'mpd')
-
-            # create mpdboot file
-            fn = os.path.join(tmpdir, 'mpdboot')
-            try:
-                if os.path.exists(fn):
-                    os.remove(fn)
-                write_file(fn, "localhost ifhn=localhost")
-            except OSError, err:
-                raise EasyBuildError("Failed to create file %s: %s", fn, err)
-
-            params.update({'mpdbf': "--file=%s" % fn})
-
-            # create nodes file
-            fn = os.path.join(tmpdir, 'nodes')
-            try:
-                if os.path.exists(fn):
-                    os.remove(fn)
-                write_file(fn, "localhost\n" * nr_ranks)
-            except OSError, err:
-                raise EasyBuildError("Failed to create file %s: %s", fn, err)
-
-            params.update({'nodesfile': "-machinefile %s" % fn})
-
         if mpi_cmd_template is None:
+
+            if mpi_family == toolchain.INTELMPI:
+
+                # for old versions of Intel MPI, we need to use MPD
+                impi_ver = self.get_software_version(self.MPI_MODULE_NAME)[0]
+                if LooseVersion(impi_ver) <= LooseVersion('4.1'):
+
+                    mpi_cmds[toolchain.INTELMPI] = "mpirun %(mpdbf)s %(nodesfile)s -np %(nr_ranks)d %(cmd)s"
+
+                    # set temporary dir for MPD
+                    # note: this needs to be kept *short*,
+                    # to avoid mpirun failing with "socket.error: AF_UNIX path too long"
+                    # exact limit is unknown, but ~20 characters seems to be OK
+                    env.setvar('I_MPI_MPD_TMPDIR', tempfile.gettempdir())
+                    mpd_tmpdir = os.environ['I_MPI_MPD_TMPDIR']
+                    if len(mpd_tmpdir) > 20:
+                        self.log.warning("$I_MPI_MPD_TMPDIR should be (very) short to avoid problems: %s", mpd_tmpdir)
+
+                    # temporary location for mpdboot and nodes files
+                    tmpdir = tempfile.mkdtemp(prefix='mpi_cmd_for-')
+
+                    # set PBS_ENVIRONMENT, so that --file option for mpdboot isn't stripped away
+                    env.setvar('PBS_ENVIRONMENT', "PBS_BATCH_MPI")
+
+                    # make sure we're always using mpd as process manager
+                    # only required for/picked up by Intel MPI v4.1 or higher, no harm done for others
+                    env.setvar('I_MPI_PROCESS_MANAGER', 'mpd')
+
+                    # create mpdboot file
+                    mpdboot = os.path.join(tmpdir, 'mpdboot')
+                    write_file(mpdboot, "localhost ifhn=localhost")
+
+                    params.update({'mpdbf': "--file=%s" % mpdboot})
+
+                    # create nodes file
+                    nodes = os.path.join(tmpdir, 'nodes')
+                    write_file(nodes, "localhost\n" * nr_ranks)
+
+                    params.update({'nodesfile': "-machinefile %s" % nodes})
+
             if mpi_family in mpi_cmds.keys():
                 mpi_cmd_template = mpi_cmds[mpi_family]
                 self.log.info("Using template MPI command '%s' for MPI family '%s'", mpi_cmd_template, mpi_family)
