@@ -1,5 +1,5 @@
 # #
-# Copyright 2012-2017 Ghent University
+# Copyright 2012-2019 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -8,7 +8,7 @@
 # Flemish Research Foundation (FWO) (http://www.fwo.be/en)
 # and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
 #
-# http://github.com/hpcugent/easybuild
+# https://github.com/easybuilders/easybuild
 #
 # EasyBuild is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -61,12 +61,12 @@ class ToolchainOptions(dict):
 
     def _add_options(self, options):
         """Add actual options dict to self"""
-        self.log.debug("_add_options: adding options %s" % options)
+        self.log.debug("Using toolchain options %s", options)
         for name, value in options.items():
             if not isinstance(value, (list, tuple,)) and len(value) == 2:
                 raise EasyBuildError("_add_options: option name %s has to be 2 element list (%s)", name, value)
             if name in self:
-                self.log.debug("_add_options: redefining previous name %s (previous value %s)", name, self.get(name))
+                self.log.devel("_add_options: redefining previous name %s (previous value %s)", name, self.get(name))
             self.__setitem__(name, value[0])
             self.description.__setitem__(name, value[1])
 
@@ -77,9 +77,9 @@ class ToolchainOptions(dict):
         for name in options_map.keys():
             if not name in self:
                 if name.startswith('_opt_'):
-                    self.log.debug("_add_options_map: no option with name %s defined, but allowed", name)
+                    self.log.devel("_add_options_map: no option with name %s defined, but allowed", name)
                 else:
-                    raise EasyBuildError("_add_options_map: no option with name %s defined", name)
+                    raise EasyBuildError("No toolchain option with name %s defined", name)
 
         self.options_map.update(options_map)
 
