@@ -1,5 +1,5 @@
 # #
-# Copyright 2012-2017 Ghent University
+# Copyright 2012-2019 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -8,7 +8,7 @@
 # Flemish Research Foundation (FWO) (http://www.fwo.be/en)
 # and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
 #
-# http://github.com/hpcugent/easybuild
+# https://github.com/easybuilders/easybuild
 #
 # EasyBuild is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@ import sys
 from unittest import TextTestRunner
 
 from easybuild.tools.config import module_classes
-from easybuild.tools.docs import avail_easyconfig_licenses_txt, gen_easyblocks_overview_rst, list_software
+from easybuild.tools.docs import avail_easyconfig_licenses, gen_easyblocks_overview_rst, list_software
 from easybuild.tools.utilities import import_available_modules
 from test.framework.utilities import EnhancedTestCase, TestLoaderFiltered, init_config
 
@@ -41,14 +41,14 @@ class DocsTest(EnhancedTestCase):
 
     def test_gen_easyblocks(self):
         """ Test gen_easyblocks_overview_rst function """
-        module = 'easybuild.easyblocks.generic'
-        modules = import_available_modules(module)
+        gen_easyblocks_pkg = 'easybuild.easyblocks.generic'
+        modules = import_available_modules(gen_easyblocks_pkg)
         common_params = {
             'ConfigureMake' : ['configopts', 'buildopts', 'installopts'],
         }
         doc_functions = ['build_step', 'configure_step', 'test_step']
 
-        eb_overview = gen_easyblocks_overview_rst(module, 'easyconfigs', common_params, doc_functions)
+        eb_overview = gen_easyblocks_overview_rst(gen_easyblocks_pkg, 'easyconfigs', common_params, doc_functions)
         ebdoc = '\n'.join(eb_overview)
 
         # extensive check for ConfigureMake easyblock
@@ -62,6 +62,17 @@ class DocsTest(EnhancedTestCase):
             '',
             "Dummy support for building and installing applications with configure/make/make install.",
             '',
+            "Extra easyconfig parameters specific to ``ConfigureMake`` easyblock",
+            "-------------------------------------------------------------------",
+            '',
+            "====================    ============    =============",
+            "easyconfig parameter    description     default value",
+            "====================    ============    =============",
+            '``test_123``            Test 1, 2, 3    ``""``       ',
+            "``test_bool``           Just a test     ``False``    ",
+            "``test_none``           Another test    ``None``     ",
+            "====================    ============    =============",
+            '',
             "Commonly used easyconfig parameters with ``ConfigureMake`` easyblock",
             "--------------------------------------------------------------------",
             '',
@@ -74,18 +85,18 @@ class DocsTest(EnhancedTestCase):
             "====================    ================================================================",
         ])
 
-        self.assertTrue(check_configuremake in ebdoc)
+        self.assertTrue(check_configuremake in ebdoc, "Found '%s' in: %s" % (check_configuremake, ebdoc))
         names = []
 
         for mod in modules:
             for name, obj in inspect.getmembers(mod, inspect.isclass):
                 eb_class = getattr(mod, name)
                 # skip imported classes that are not easyblocks
-                if eb_class.__module__.startswith(module):
+                if eb_class.__module__.startswith(gen_easyblocks_pkg):
                     self.assertTrue(name in ebdoc)
                     names.append(name)
 
-        toc = [":ref:`" + n + "`" for n in sorted(names)]
+        toc = [":ref:`" + n + "`" for n in sorted(set(names))]
         pattern = " - ".join(toc)
 
         regex = re.compile(pattern)
@@ -93,10 +104,14 @@ class DocsTest(EnhancedTestCase):
 
     def test_license_docs(self):
         """Test license_documentation function."""
-        lic_docs = avail_easyconfig_licenses_txt()
+        lic_docs = avail_easyconfig_licenses(output_format='txt')
         gplv3 = "GPLv3: The GNU General Public License"
         self.assertTrue(gplv3 in lic_docs, "%s found in: %s" % (gplv3, lic_docs))
 
+        lic_docs = avail_easyconfig_licenses(output_format='rst')
+        regex = re.compile("^``GPLv3``\s*The GNU General Public License", re.M)
+        self.assertTrue(regex.search(lic_docs), "%s found in: %s" % (regex.pattern, lic_docs))
+
     def test_list_software(self):
         """Test list_software* functions."""
         build_options = {
@@ -130,7 +145,7 @@ class DocsTest(EnhancedTestCase):
             r'homepage: http://www.gzip.org/',
             r'',
             r"  \* gzip v1.4: GCC/4.6.3, dummy",
-            r"  \* gzip v1.5: goolf/1.4.10, ictce/4.1.13",
+            r"  \* gzip v1.5: foss/2018a, intel/2018a",
             '',
         ]))
         txt = list_software(output_format='txt', detailed=True)
@@ -198,12 +213,12 @@ class DocsTest(EnhancedTestCase):
             r'',
             r'\*homepage\*: http://www.gzip.org/',
             r'',
-            r'=======    ==================================',
-            r'version    toolchain                         ',
-            r'=======    ==================================',
-            r'``1.4``    ``GCC/4.6.3``, ``dummy``          ',
-            r'``1.5``    ``goolf/1.4.10``, ``ictce/4.1.13``',
-            r'=======    ==================================',
+            r'=======    ===============================',
+            r'version    toolchain                      ',
+            r'=======    ===============================',
+            r'``1.4``    ``GCC/4.6.3``, ``dummy``       ',
+            r'``1.5``    ``foss/2018a``, ``intel/2018a``',
+            r'=======    ===============================',
         ]))
         txt = list_software(output_format='rst', detailed=True)
         self.assertTrue(expected.match(txt), "Pattern '%s' found in: %s" % (expected.pattern, txt))
@@ -233,15 +248,15 @@ class DocsTest(EnhancedTestCase):
         expected = [
             '* toy',
             '',
-            'Toy C program.',
+            'Toy C program, 100% toy.',
             '',
-            'homepage: http://hpcugent.github.com/easybuild',
+            'homepage: https://easybuilders.github.io/easybuild',
             '',
-            "  * toy v0.0: dummy",
+            "  * toy v0.0: dummy, gompi/2018a",
             "  * toy v0.0 (versionsuffix: '-deps'): dummy",
             "  * toy v0.0 (versionsuffix: '-iter'): dummy",
             "  * toy v0.0 (versionsuffix: '-multiple'): dummy",
-            "  * toy v0.0 (versionsuffix: '-test'): gompi/1.3.12",
+            "  * toy v0.0 (versionsuffix: '-test'): gompi/2018a",
         ]
         txt = list_software(output_format='txt', detailed=True)
         lines = txt.split('\n')
@@ -252,19 +267,19 @@ class DocsTest(EnhancedTestCase):
             '*toy*',
             '+++++',
             '',
-            'Toy C program.',
+            'Toy C program, 100% toy.',
             '',
-            '*homepage*: http://hpcugent.github.com/easybuild',
+            '*homepage*: https://easybuilders.github.io/easybuild',
             '',
-            '=======    =============    ================',
-            'version    versionsuffix    toolchain       ',
-            '=======    =============    ================',
-            '``0.0``    ````             ``dummy``       ',
-            '``0.0``    ``-deps``        ``dummy``       ',
-            '``0.0``    ``-iter``        ``dummy``       ',
-            '``0.0``    ``-multiple``    ``dummy``       ',
-            '``0.0``    ``-test``        ``gompi/1.3.12``',
-            '=======    =============    ================',
+            '=======    =============    ==========================',
+            'version    versionsuffix    toolchain                 ',
+            '=======    =============    ==========================',
+            '``0.0``                     ``dummy``, ``gompi/2018a``',
+            '``0.0``    ``-deps``        ``dummy``                 ',
+            '``0.0``    ``-iter``        ``dummy``                 ',
+            '``0.0``    ``-multiple``    ``dummy``                 ',
+            '``0.0``    ``-test``        ``gompi/2018a``           ',
+            '=======    =============    ==========================',
         ]
         txt = list_software(output_format='rst', detailed=True)
         lines = txt.split('\n')
