@@ -1,5 +1,5 @@
 ##
-# Copyright 2015-2017 Ghent University
+# Copyright 2015-2019 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -8,7 +8,7 @@
 # Flemish Research Foundation (FWO) (http://www.fwo.be/en)
 # and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
 #
-# http://github.com/hpcugent/easybuild
+# https://github.com/easybuilders/easybuild
 #
 # EasyBuild is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -33,7 +33,7 @@ import sys
 from test.framework.utilities import EnhancedTestCase, TestLoaderFiltered
 from unittest import TextTestRunner
 
-import vsc
+import vsc.utils.generaloption
 
 import easybuild.framework
 from easybuild.tools.build_log import EasyBuildError
@@ -46,11 +46,12 @@ class GeneralTest(EnhancedTestCase):
 
     def test_vsc_location(self):
         """Make sure location of imported vsc module is not the framework itself."""
-        # cfr. https://github.com/hpcugent/easybuild-framework/pull/1160
+        # cfr. https://github.com/easybuilders/easybuild-framework/pull/1160
         # easybuild.framework.__file__ provides location to <prefix>/easybuild/framework/__init__.py
         framework_loc = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(easybuild.framework.__file__))))
-        # vsc.__file__ provides location to <prefix>/vsc/__init__.py
-        vsc_loc = os.path.dirname(os.path.dirname(os.path.abspath(vsc.__file__)))
+        # vsc.utils.generaloption.__file__ provides location to <prefix>/vsc/utils/generaloption/__init__.py
+        generaloption_loc = os.path.abspath(vsc.utils.generaloption.__file__)
+        vsc_loc = os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(generaloption_loc))))
         # make sure vsc is being imported from outside of framework
         msg = "vsc-base is not provided by EasyBuild framework itself, found location: %s" % vsc_loc
         self.assertFalse(os.path.samefile(framework_loc, vsc_loc), msg)
@@ -81,6 +82,12 @@ class GeneralTest(EnhancedTestCase):
 
         foo()
 
+        @only_if_module_is_available(('nosuchmoduleoutthere', 'easybuild'))
+        def foo2():
+            pass
+
+        foo2()
+
         @only_if_module_is_available('nosuchmoduleoutthere', pkgname='nosuchpkg')
         def bar():
             pass
@@ -88,6 +95,13 @@ class GeneralTest(EnhancedTestCase):
         err_pat = "required module 'nosuchmoduleoutthere' is not available.*package nosuchpkg.*pypi/nosuchpkg"
         self.assertErrorRegex(EasyBuildError, err_pat, bar)
 
+        @only_if_module_is_available(('nosuchmodule', 'anothernosuchmodule'))
+        def bar2():
+            pass
+
+        err_pat = "ImportError: None of the specified modules nosuchmodule, anothernosuchmodule is available"
+        self.assertErrorRegex(EasyBuildError, err_pat, bar2)
+
         class Foo():
             @only_if_module_is_available('thisdoesnotexist', url='http://example.com')
             def foobar(self):
