@@ -1,5 +1,5 @@
 ##
-# Copyright 2009-2017 Ghent University
+# Copyright 2009-2019 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -8,7 +8,7 @@
 # Flemish Research Foundation (FWO) (http://www.fwo.be/en)
 # and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
 #
-# http://github.com/hpcugent/easybuild
+# https://github.com/easybuilders/easybuild
 #
 # EasyBuild is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -32,16 +32,23 @@ import os
 import platform
 import shutil
 
-from easybuild.framework.easyblock import EasyBlock
+from easybuild.framework.extensioneasyblock import ExtensionEasyBlock
 from easybuild.tools.build_log import EasyBuildError
+from easybuild.tools.environment import setvar
 from easybuild.tools.filetools import mkdir
 from easybuild.tools.modules import get_software_root, get_software_version
 from easybuild.tools.run import run_cmd
 
 
-class EB_toy(EasyBlock):
+class EB_toy(ExtensionEasyBlock):
     """Support for building/installing toy."""
 
+    def __init__(self, *args, **kwargs):
+        """Constructor"""
+        super(EB_toy, self).__init__(*args, **kwargs)
+
+        setvar('TOY', '%s-%s' % (self.name, self.version))
+
     def prepare_for_extensions(self):
         """
         Prepare for installing toy extensions.
@@ -62,14 +69,18 @@ class EB_toy(EasyBlock):
         if os.path.exists("%s.source" % name):
             os.rename('%s.source' % name, '%s.c' % name)
 
-    def build_step(self, name=None):
+    def build_step(self, name=None, buildopts=None):
         """Build toy."""
+
+        if buildopts is None:
+            buildopts = self.cfg['buildopts']
+
         if name is None:
             name = self.name
         run_cmd('%(prebuildopts)s gcc %(name)s.c -o %(name)s %(buildopts)s' % {
             'name': name,
             'prebuildopts': self.cfg['prebuildopts'],
-            'buildopts': self.cfg['buildopts'],
+            'buildopts': buildopts,
         })
 
     def install_step(self, name=None):
@@ -87,3 +98,16 @@ class EB_toy(EasyBlock):
         f = open(os.path.join(libdir, 'lib%s.a' % name), 'w')
         f.write(name.upper())
         f.close()
+
+    def run(self):
+        """Install toy as extension."""
+        super(EB_toy, self).run(unpack_src=True)
+        self.configure_step()
+        self.build_step()
+        self.install_step()
+
+    def make_module_extra(self):
+        """Extra stuff for toy module"""
+        txt = super(EB_toy, self).make_module_extra()
+        txt += self.module_generator.set_environment('TOY', os.getenv('TOY', '<TOY_env_var_not_defined>'))
+        return txt
