@@ -1,5 +1,5 @@
 # #
-# Copyright 2014 Ghent University
+# Copyright 2014-2019 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -8,7 +8,7 @@
 # Flemish Research Foundation (FWO) (http://www.fwo.be/en)
 # and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
 #
-# http://github.com/hpcugent/easybuild
+# https://github.com/easybuilders/easybuild
 #
 # EasyBuild is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@ from test.framework.utilities import EnhancedTestCase, TestLoaderFiltered
 from unittest import TextTestRunner
 
 import setuptools
-import vsc
+import vsc.utils.generaloption
 
 import easybuild.framework
 from easybuild.framework.easyconfig.easyconfig import EasyConfig
@@ -53,7 +53,8 @@ class ScriptsTest(EnhancedTestCase):
 
         # make sure setuptools, vsc-base and easybuild-framework are included in $PYTHONPATH (so scripts can pick it up)
         setuptools_loc = os.path.dirname(os.path.dirname(setuptools.__file__))
-        vsc_loc = os.path.dirname(os.path.dirname(os.path.abspath(vsc.__file__)))
+        generaloption_loc = os.path.abspath(vsc.utils.generaloption.__file__)
+        vsc_loc = os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(generaloption_loc))))
         framework_loc = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(easybuild.framework.__file__))))
         pythonpath = os.environ.get('PYTHONPATH', '')
         os.environ['PYTHONPATH'] = os.pathsep.join([setuptools_loc, vsc_loc, framework_loc, pythonpath])
@@ -84,15 +85,15 @@ class ScriptsTest(EnhancedTestCase):
         out, ec = run_cmd(cmd, simple=False)
 
         # make sure output is kind of what we expect it to be
-        regex = r"Supported Packages \(26 "
+        regex = r"Supported Packages \(32 "
         self.assertTrue(re.search(regex, out), "Pattern '%s' found in output: %s" % (regex, out))
         per_letter = {
-            'B': '1',  # bzip2
+            'B': '2',  # binutils, bzip2
             'C': '2',  # CrayCCE, CUDA
-            'F': '1',  # FFTW
-            'G': '6',  # GCC, GCCcore, gmvapich2, gompi, goolf, gzip
+            'F': '3',  # foss, fosscuda, FFTW
+            'G': '9',  # GCC, GCCcore, gcccuda, gmvapich2, golf, golfc, gompic, gompi, gzip
             'H': '1',  # hwloc
-            'I': '8',  # icc, iccifort, iccifortcuda, ictce, ifort, iimpi, imkl, impi
+            'I': '8',  # icc, iccifort, iccifortcuda, intel, ifort, iimpi, imkl, impi
             'M': '1',  # MVAPICH2
             'O': '2',  # OpenMPI, OpenBLAS
             'P': '1',  # Python
@@ -101,13 +102,14 @@ class ScriptsTest(EnhancedTestCase):
         }
         self.assertTrue(' - '.join(["[%(l)s](#%(l)s)" % {'l': l} for l in sorted(per_letter.keys())]))
         for key, val in per_letter.items():
-            self.assertTrue(re.search(r"### %(l)s \(%(n)s packages\) <a name='%(l)s'/>" % {'l': key, 'n': val}, out))
+            regex = re.compile(r"### %(l)s \(%(n)s packages\) <a name='%(l)s'/>" % {'l': key, 'n': val})
+            self.assertTrue(regex.search(out), "Pattern '%s' found in: %s" % (regex.pattern, out))
 
-        software = ['FFTW', 'GCC', 'gompi', 'goolf', 'gzip', 'hwloc', 'OpenMPI', 'OpenBLAS', 'ScaLAPACK', 'toy']
+        software = ['FFTW', 'foss', 'GCC', 'gompi', 'gzip', 'hwloc', 'OpenMPI', 'OpenBLAS', 'ScaLAPACK', 'toy']
         for soft in software:
             letter = soft[0].lower()
             pattern = r"^\*.*logo[\s\S]*easyconfigs/%(l)s/%(s)s\)[\s\S]*%(s)s.*\n" % {'l': letter, 's': soft}
-            self.assertTrue(re.search(pattern, out, re.M))
+            self.assertTrue(re.search(pattern, out, re.M), "Pattern '%s' found in: %s" % (pattern, out))
 
         shutil.rmtree(tmpdir)
         os.environ['PYTHONPATH'] = pythonpath
@@ -217,9 +219,11 @@ class ScriptsTest(EnhancedTestCase):
         self.assertTrue(EasyConfig(None, rawtxt=new_ec_txt))
         self.assertEqual(read_file('%s.bk' % broken_ec), broken_ec_txt)
 
+
 def suite():
     """ returns all the testcases in this module """
     return TestLoaderFiltered().loadTestsFromTestCase(ScriptsTest, sys.argv[1:])
 
+
 if __name__ == '__main__':
     TextTestRunner(verbosity=1).run(suite())
