@@ -1,6 +1,6 @@
 #!/usr/bin/python
 # #
-# Copyright 2012-2017 Ghent University
+# Copyright 2012-2019 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -9,7 +9,7 @@
 # Flemish Research Foundation (FWO) (http://www.fwo.be/en)
 # and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
 #
-# http://github.com/hpcugent/easybuild
+# https://github.com/easybuilders/easybuild
 #
 # EasyBuild is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -41,21 +41,10 @@ from vsc.utils import fancylogger
 from easybuild.tools.build_log import EasyBuildError
 from easybuild.tools.options import set_tmpdir
 
-# set plain text key ring to be used, so a GitHub token stored in it can be obtained without having to provide a password
-try:
-    import keyring
-    keyring.set_keyring(keyring.backends.file.PlaintextKeyring())
-except (ImportError, AttributeError):
-    pass
-
-# disable all logging to significantly speed up tests
-fancylogger.disableDefaultHandlers()
-fancylogger.setLogLevelError()
-
-# toolkit should be first to allow hacks to work
 import test.framework.asyncprocess as a
 import test.framework.build_log as bl
 import test.framework.config as c
+import test.framework.containers as ct
 import test.framework.easyblock as b
 import test.framework.easyconfig as e
 import test.framework.easyconfigparser as ep
@@ -68,8 +57,10 @@ import test.framework.filetools as f
 import test.framework.format_convert as f_c
 import test.framework.general as gen
 import test.framework.github as g
+import test.framework.hooks as h
 import test.framework.include as i
-import test.framework.license as l
+import test.framework.lib as lib
+import test.framework.license as lic
 import test.framework.module_generator as mg
 import test.framework.modules as m
 import test.framework.modulestool as mt
@@ -90,6 +81,26 @@ import test.framework.tweak as tw
 import test.framework.variables as v
 import test.framework.yeb as y
 
+# set plain text key ring to be used,
+# so a GitHub token stored in it can be obtained without having to provide a password
+try:
+    # with recent versions of keyring, PlaintextKeyring comes from keyrings.alt
+    import keyring
+    from keyrings.alt.file import PlaintextKeyring
+    keyring.set_keyring(PlaintextKeyring())
+except ImportError as err:
+    try:
+        # with old versions of keyring, PlaintextKeyring comes from keyring.backends
+        import keyring
+        from keyring.backends.file import PlaintextKeyring
+        keyring.set_keyring(PlaintextKeyring())
+    except ImportError as err:
+        pass
+
+# disable all logging to significantly speed up tests
+fancylogger.disableDefaultHandlers()
+fancylogger.setLogLevelError()
+
 
 # make sure temporary files can be created/used
 try:
@@ -107,8 +118,8 @@ log = fancylogger.getLogger()
 
 # call suite() for each module and then run them all
 # note: make sure the options unit tests run first, to avoid running some of them with a readily initialized config
-tests = [gen, bl, o, r, ef, ev, ebco, ep, e, mg, m, mt, f, run, a, robot, b, v, g, tcv, tc, t, c, s, l, f_c, sc,
-         tw, p, i, pkg, d, env, et, y, st]
+tests = [gen, bl, o, r, ef, ev, ebco, ep, e, mg, m, mt, f, run, a, robot, b, v, g, tcv, tc, t, c, s, lic, f_c, sc,
+         tw, p, i, pkg, d, env, et, y, st, h, ct, lib]
 
 SUITE = unittest.TestSuite([x.suite() for x in tests])
 
@@ -130,5 +141,5 @@ if not res.wasSuccessful():
     print "Log available at %s" % log_fn, xml_msg
     sys.exit(2)
 else:
-    for f in glob.glob('%s*' % log_fn):
-        os.remove(f)
+    for fn in glob.glob('%s*' % log_fn):
+        os.remove(fn)
